@@ -77,6 +77,7 @@ from daft.expressions import Expression, col, element, list_, lit, interval, str
 from daft.io import (
     DataCatalogTable,
     DataCatalogType,
+    IOConfig,
     from_glob_path,
     _range as range,
     read_csv,
@@ -101,7 +102,9 @@ from daft.session import (
     create_table_if_not_exists,
     create_temp_table,
     current_catalog,
+    current_model,
     current_namespace,
+    current_provider,
     current_session,
     detach_catalog,
     detach_table,
@@ -116,17 +119,22 @@ from daft.session import (
     list_tables,
     read_table,
     set_catalog,
+    set_model,
     set_namespace,
+    set_provider,
     set_session,
     write_table,
     attach_function,
     detach_function,
 )
 from daft.sql import sql, sql_expr
-from daft.udf import udf, _DaftFunc as func
+from daft.udf import udf, _DaftFuncDecorator as func
 from daft.viz import register_viz_hook
 from daft.window import Window
 
+import daft.context as context
+import daft.io as io
+
 to_struct = Expression.to_struct
 
 __all__ = [
@@ -136,6 +144,7 @@ __all__ = [
     "DataFrame",
     "DataType",
     "Expression",
+    "IOConfig",
     "Identifier",
     "ImageFormat",
     "ImageMode",
@@ -152,13 +161,16 @@ __all__ = [
     "attach_table",
     "coalesce",
     "col",
+    "context",
     "create_namespace",
     "create_namespace_if_not_exists",
     "create_table",
     "create_table_if_not_exists",
     "create_temp_table",
     "current_catalog",
+    "current_model",
     "current_namespace",
+    "current_provider",
     "current_session",
     "detach_catalog",
     "detach_function",
@@ -181,6 +193,7 @@ __all__ = [
     "has_namespace",
     "has_table",
     "interval",
+    "io",
     "list_",
     "list_catalogs",
     "list_tables",
@@ -201,8 +214,10 @@ __all__ = [
     "register_viz_hook",
     "set_catalog",
     "set_execution_config",
+    "set_model",
     "set_namespace",
     "set_planning_config",
+    "set_provider",
     "set_session",
     "sql",
     "sql_expr",
