@@ -184,6 +184,10 @@ class LogicalPlanBuilder:
         builder = self._builder.limit(num_rows, eager)
         return LogicalPlanBuilder(builder)
 
+    def offset(self, num_rows: int) -> LogicalPlanBuilder:
+        builder = self._builder.offset(num_rows)
+        return LogicalPlanBuilder(builder)
+
     def shard(self, strategy: str, world_size: int, rank: int) -> LogicalPlanBuilder:
         builder = self._builder.shard(strategy, world_size, rank)
         return LogicalPlanBuilder(builder)
@@ -250,6 +254,10 @@ class LogicalPlanBuilder:
         builder = self._builder.into_partitions(num_partitions)
         return LogicalPlanBuilder(builder)
 
+    def into_batches(self, batch_size: int) -> LogicalPlanBuilder:
+        builder = self._builder.into_batches(batch_size)
+        return LogicalPlanBuilder(builder)
+
     def agg(
         self,
         to_agg: list[Expression],
