@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import logging
+import os
 from dataclasses import dataclass
 from typing import TYPE_CHECKING
 
@@ -44,9 +45,11 @@ class RaySwordfishActor:
     It is a stateless, async actor, and can run multiple plans concurrently and is able to retry itself and it's tasks.
     """
 
-    def __init__(self, num_worker_threads: int) -> None:
+    def __init__(self, num_cpus: int, num_gpus: int) -> None:
+        if num_gpus > 0:
+            os.environ["CUDA_VISIBLE_DEVICES"] = ",".join(str(i) for i in range(num_gpus))
         # Configure the number of worker threads for swordfish, according to the number of CPUs visible to ray.
-        set_compute_runtime_num_worker_threads(num_worker_threads)
+        set_compute_runtime_num_worker_threads(num_cpus)
 
     async def run_plan(
         self,
@@ -156,7 +159,10 @@ def start_ray_workers(existing_worker_ids: list[str]) -> list[RaySwordfishWorker
                     node_id=node["NodeID"],
                     soft=False,
                 ),
-            ).remote(num_worker_threads=int(node["Resources"]["CPU"]))
+            ).remote(
+                num_cpus=int(node["Resources"]["CPU"]),
+                num_gpus=int(node["Resources"].get("GPU", 0)),
+            )
             actor_handle = RaySwordfishActorHandle(actor)
             handles.append(
                 RaySwordfishWorker(
@@ -171,25 +177,22 @@ def start_ray_workers(existing_worker_ids: list[str]) -> list[RaySwordfishWorker
     return handles
 
 
-def try_autoscale(num_cpus: int) -> None:
+def try_autoscale(bundles: list[dict[str, int]]) -> None:
     from ray.autoscaler.sdk import request_resources
 
     request_resources(
-        num_cpus=num_cpus,
+        bundles=bundles,
     )
 
 
-class FlotillaRunnerCore:
-    """Core functionality for running distributed physical plans with Flotilla.
-
-    This class contains the business logic for managing plans and their execution,
-    separate from the Ray actor wrapper.
-    """
-
-    def __init__(self, on_actor: bool = False) -> None:
+@ray.remote(
+    num_cpus=0,
+)
+class RemoteFlotillaRunner:
+    def __init__(self) -> None:
         self.curr_plans: dict[str, DistributedPhysicalPlan] = {}
         self.curr_result_gens: dict[str, AsyncIterator[RayPartitionRef]] = {}
-        self.plan_runner = DistributedPhysicalPlanRunner(on_actor)
+        self.plan_runner = DistributedPhysicalPlanRunner()
 
     def run_plan(
         self,
@@ -229,58 +232,7 @@ class FlotillaRunnerCore:
         return materialized_result
 
 
-class LocalFlotillaRunner:
-    """Local wrapper around FlotillaPlanRunnerCore.
-
-    This wrapper provides the same interface as FlotillaPlanRunner but without
-    Ray actor overhead, useful for local testing or when distributed execution
-    is not needed.
-    """
-
-    def __init__(self) -> None:
-        self.loop = asyncio.new_event_loop()
-        self.core = self.loop.run_until_complete(self._make_runner())
-
-    async def _make_runner(self) -> FlotillaRunnerCore:
-        return FlotillaRunnerCore(on_actor=False)
-
-    def run_plan(
-        self,
-        plan: DistributedPhysicalPlan,
-        partition_sets: dict[str, PartitionSet[ray.ObjectRef]],
-    ) -> None:
-        self.core.run_plan(plan, partition_sets)
-
-    def get_next_partition(self, plan_id: str) -> RayMaterializedResult | None:
-        """Synchronous version of get_next_partition that internally uses asyncio."""
-        return self.loop.run_until_complete(self.core.get_next_partition(plan_id))
-
-
-@ray.remote(
-    num_cpus=0,
-)
-class RemoteFlotillaRunner:
-    """Ray actor wrapper around FlotillaPlanRunnerCore.
-
-    This actor provides the distributed interface for running plans,
-    while delegating the actual work to the core class.
-    """
-
-    def __init__(self) -> None:
-        self.core = FlotillaRunnerCore(on_actor=True)
-
-    def run_plan(
-        self,
-        plan: DistributedPhysicalPlan,
-        partition_sets: dict[str, PartitionSet[ray.ObjectRef]],
-    ) -> None:
-        self.core.run_plan(plan, partition_sets)
-
-    async def get_next_partition(self, plan_id: str) -> RayMaterializedResult | None:
-        return await self.core.get_next_partition(plan_id)
-
-
-FLOTILLA_RUNER_NAMESPACE = "daft"
+FLOTILLA_RUNNER_NAMESPACE = "daft"
 FLOTILLA_RUNNER_NAME = "flotilla-plan-runner"
 
 
@@ -298,39 +250,31 @@ def get_head_node_id() -> str | None:
 class FlotillaRunner:
     """FlotillaRunner is a wrapper around FlotillaRunnerCore that provides a Ray actor interface."""
 
-    def __init__(self, use_actor: bool = False) -> None:
-        self.use_actor = use_actor
-        if self.use_actor:
-            head_node_id = get_head_node_id()
-            self.runner = RemoteFlotillaRunner.options(  # type: ignore
-                name=FLOTILLA_RUNNER_NAME,
-                namespace=FLOTILLA_RUNER_NAMESPACE,
-                get_if_exists=True,
-                scheduling_strategy=ray.util.scheduling_strategies.NodeAffinitySchedulingStrategy(
+    def __init__(self) -> None:
+        head_node_id = get_head_node_id()
+        self.runner = RemoteFlotillaRunner.options(  # type: ignore
+            name=FLOTILLA_RUNNER_NAME,
+            namespace=FLOTILLA_RUNNER_NAMESPACE,
+            get_if_exists=True,
+            scheduling_strategy=(
+                ray.util.scheduling_strategies.NodeAffinitySchedulingStrategy(
                     node_id=head_node_id,
                     soft=False,
                 )
                 if head_node_id is not None
-                else "DEFAULT",
-            ).remote()
-        else:
-            self.runner = LocalFlotillaRunner()
+                else "DEFAULT"
+            ),
+        ).remote()
 
     def stream_plan(
-        self, plan: DistributedPhysicalPlan, partition_sets: dict[str, PartitionSet[ray.ObjectRef]]
+        self,
+        plan: DistributedPhysicalPlan,
+        partition_sets: dict[str, PartitionSet[ray.ObjectRef]],
     ) -> Iterator[RayMaterializedResult]:
         plan_id = plan.id()
-        if self.use_actor:
-            ray.get(self.runner.run_plan.remote(plan, partition_sets))
-            while True:
-                materialized_result = ray.get(self.runner.get_next_partition.remote(plan_id))
-                if materialized_result is None:
-                    break
-                yield materialized_result
-        else:
-            self.runner.run_plan(plan, partition_sets)
-            while True:
-                materialized_result = self.runner.get_next_partition(plan_id)
-                if materialized_result is None:
-                    break
-                yield materialized_result
+        ray.get(self.runner.run_plan.remote(plan, partition_sets))
+        while True:
+            materialized_result = ray.get(self.runner.get_next_partition.remote(plan_id))
+            if materialized_result is None:
+                break
+            yield materialized_result
