@@ -23,7 +23,9 @@ __all__ = [
     "create_table_if_not_exists",
     "create_temp_table",
     "current_catalog",
+    "current_model",
     "current_namespace",
+    "current_provider",
     "current_session",
     "detach_catalog",
     "detach_function",
@@ -40,7 +42,9 @@ __all__ = [
     "list_tables",
     "read_table",
     "set_catalog",
+    "set_model",
     "set_namespace",
+    "set_provider",
     "set_session",
     "write_table",
 ]
@@ -341,6 +345,22 @@ class Session:
         ident = self._session.current_namespace()
         return Identifier._from_pyidentifier(ident) if ident else None
 
+    def current_provider(self) -> str | None:
+        """Get the session's current provider or None.
+
+        Returns:
+            str: the session's default provider identifier
+        """
+        return self._session.current_provider()
+
+    def current_model(self) -> str | None:
+        """Get the session's current model or None.
+
+        Returns:
+            str: the session's default model identifier
+        """
+        return self._session.current_model()
+
     ###
     # get_*
     ###
@@ -473,6 +493,22 @@ class Session:
             identifier = Identifier.from_str(identifier)
         self._session.set_namespace(identifier._ident if identifier else None)
 
+    def set_provider(self, identifier: str | None) -> None:
+        """Set the default model provider.
+
+        Args:
+            identifier (str | None): provider identifier string or None.
+        """
+        self._session.set_provider(identifier)
+
+    def set_model(self, identifier: str | None) -> None:
+        """Set the default model type.
+
+        Args:
+            identifier (str | None): model identifier string.
+        """
+        self._session.set_model(identifier)
+
     ###
     # write_*
     ###
@@ -611,15 +647,25 @@ def drop_table(identifier: Identifier | str) -> None:
 
 
 def current_catalog() -> Catalog | None:
-    """Returns the session's current catalog or None."""
+    """Returns the active session's current catalog or None."""
     return _session().current_catalog()
 
 
 def current_namespace() -> Identifier | None:
-    """Returns the session's current namespace or None."""
+    """Returns the active session's current namespace or None."""
     return _session().current_namespace()
 
 
+def current_model() -> str | None:
+    """Returns the active session's current model or None."""
+    return _session().current_model()
+
+
+def current_provider() -> str | None:
+    """Returns the active session's current provider or None."""
+    return _session().current_provider()
+
+
 def current_session() -> Session:
     """Returns the global context's current session."""
     return _session()
@@ -716,10 +762,20 @@ def set_catalog(identifier: str | None) -> None:
 
 
 def set_namespace(identifier: Identifier | str | None) -> None:
-    """Set the given namespace as current_namespace for the current session."""
+    """Set the given namespace as current_namespace for the active session."""
     _session().set_namespace(identifier)
 
 
+def set_provider(identifier: str | None) -> None:
+    """Set the given provider as current_provider for the active session."""
+    _session().set_provider(identifier)
+
+
+def set_model(identifier: str | None) -> None:
+    """Set the given model as current_model for the active session."""
+    _session().set_model(identifier)
+
+
 def set_session(session: Session) -> None:
     """Sets the global context's current session."""
     # Consider registering into the global context.
