@@ -7,7 +7,6 @@ use common_scan_info::ScanState;
 use common_treenode::{TreeNode, TreeNodeRecursion, TreeNodeVisitor};
 use daft_dsl::{
     expr::bound_expr::{BoundAggExpr, BoundExpr, BoundWindowExpr},
-    functions::python::{get_resource_request, try_get_batch_size_from_udf},
     resolved_col,
 };
 use daft_logical_plan::{partitioning::RepartitionSpec, LogicalPlan, LogicalPlanRef, SourceInfo};
@@ -16,11 +15,11 @@ use daft_physical_plan::extract_agg_expr;
 use crate::{
     pipeline_node::{
         concat::ConcatNode, distinct::DistinctNode, explode::ExplodeNode, filter::FilterNode,
-        gather::GatherNode, in_memory_source::InMemorySourceNode, limit::LimitNode,
-        monotonically_increasing_id::MonotonicallyIncreasingIdNode, project::ProjectNode,
-        repartition::RepartitionNode, sample::SampleNode, scan_source::ScanSourceNode,
-        sink::SinkNode, sort::SortNode, top_n::TopNNode, unpivot::UnpivotNode, window::WindowNode,
-        DistributedPipelineNode, NodeID,
+        gather::GatherNode, in_memory_source::InMemorySourceNode, into_batches::IntoBatchesNode,
+        limit::LimitNode, monotonically_increasing_id::MonotonicallyIncreasingIdNode,
+        project::ProjectNode, repartition::RepartitionNode, sample::SampleNode,
+        scan_source::ScanSourceNode, sink::SinkNode, sort::SortNode, top_n::TopNNode, udf::UDFNode,
+        unpivot::UnpivotNode, window::WindowNode, DistributedPipelineNode, NodeID,
     },
     stage::StageConfig,
 };
@@ -81,16 +80,22 @@ impl LogicalPlanToPipelineNodeTranslator {
         logical_node_id: Option<NodeID>,
         input_node: Arc<dyn DistributedPipelineNode>,
         partition_cols: Vec<BoundExpr>,
+        num_partitions: Option<usize>,
     ) -> Arc<dyn DistributedPipelineNode> {
         if partition_cols.is_empty() {
             self.gen_gather_node(logical_node_id, input_node)
         } else {
+            let repartition_spec = daft_logical_plan::partitioning::RepartitionSpec::Hash(
+                daft_logical_plan::partitioning::HashRepartitionConfig::new(
+                    num_partitions,
+                    partition_cols.into_iter().map(|e| e.into()).collect(),
+                ),
+            );
             RepartitionNode::new(
                 self.get_next_pipeline_node_id(),
                 logical_node_id,
                 &self.stage_config,
-                partition_cols,
-                None,
+                repartition_spec,
                 input_node.config().schema.clone(),
                 input_node,
             )
@@ -136,26 +141,24 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     SourceInfo::PlaceHolder(_) => unreachable!("PlaceHolder should not be present in the logical plan for pipeline node translation"),
                 }
             }
-            LogicalPlan::ActorPoolProject(actor_pool_project) => {
+            LogicalPlan::UDFProject(udf) if udf.is_actor_pool_udf() => {
                 #[cfg(feature = "python")]
                 {
-                    let batch_size = try_get_batch_size_from_udf(&actor_pool_project.projection)?;
-                    let memory_request = get_resource_request(&actor_pool_project.projection)
-                        .and_then(|req| req.memory_bytes())
-                        .map(|m| m as u64)
-                        .unwrap_or(0);
-                    let projection = BoundExpr::bind_all(
-                        &actor_pool_project.projection,
-                        &actor_pool_project.input.schema(),
-                    )?;
+                    let projection = udf
+                        .passthrough_columns
+                        .iter()
+                        .chain(std::iter::once(&udf.project.clone()))
+                        .cloned()
+                        .collect::<Vec<_>>();
+                    let projection =
+                        BoundExpr::bind_all(projection.as_slice(), &udf.input.schema())?;
                     crate::pipeline_node::actor_udf::ActorUDF::new(
                         self.get_next_pipeline_node_id(),
                         logical_node_id,
                         &self.stage_config,
                         projection,
-                        batch_size,
-                        memory_request,
-                        actor_pool_project.projected_schema.clone(),
+                        udf.udf_properties.clone(),
+                        udf.projected_schema.clone(),
                         self.curr_node.pop().unwrap(),
                     )?
                     .arced()
@@ -165,6 +168,23 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     panic!("ActorUDF is not supported without Python feature")
                 }
             }
+            LogicalPlan::UDFProject(udf) => {
+                let project = BoundExpr::try_new(udf.project.clone(), &udf.input.schema())?;
+                let passthrough_columns =
+                    BoundExpr::bind_all(&udf.passthrough_columns, &udf.input.schema())?;
+
+                UDFNode::new(
+                    self.get_next_pipeline_node_id(),
+                    logical_node_id,
+                    &self.stage_config,
+                    project,
+                    passthrough_columns,
+                    udf.udf_properties.clone(),
+                    node.schema(),
+                    self.curr_node.pop().unwrap(),
+                )
+                .arced()
+            }
             LogicalPlan::Filter(filter) => {
                 let predicate =
                     BoundExpr::try_new(filter.predicate.clone(), &filter.input.schema())?;
@@ -178,11 +198,21 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                 )
                 .arced()
             }
+            LogicalPlan::IntoBatches(into_batches) => IntoBatchesNode::new(
+                self.get_next_pipeline_node_id(),
+                logical_node_id,
+                &self.stage_config,
+                into_batches.batch_size,
+                node.schema(),
+                self.curr_node.pop().unwrap(),
+            )
+            .arced(),
             LogicalPlan::Limit(limit) => Arc::new(LimitNode::new(
                 self.get_next_pipeline_node_id(),
                 logical_node_id,
                 &self.stage_config,
                 limit.limit as usize,
+                limit.offset.map(|x| x as usize),
                 node.schema(),
                 self.curr_node.pop().unwrap(),
             )),
@@ -237,16 +267,19 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                 self.curr_node.pop().unwrap(),
             )
             .arced(),
-            LogicalPlan::Sink(sink) => SinkNode::new(
-                self.get_next_pipeline_node_id(),
-                logical_node_id,
-                &self.stage_config,
-                sink.sink_info.clone(),
-                sink.schema.clone(),
-                sink.input.schema(),
-                self.curr_node.pop().unwrap(),
-            )
-            .arced(),
+            LogicalPlan::Sink(sink) => {
+                let sink_info = sink.sink_info.bind(&sink.input.schema())?;
+                SinkNode::new(
+                    self.get_next_pipeline_node_id(),
+                    logical_node_id,
+                    &self.stage_config,
+                    sink_info.into(),
+                    sink.schema.clone(),
+                    sink.input.schema(),
+                    self.curr_node.pop().unwrap(),
+                )
+                .arced()
+            }
             LogicalPlan::MonotonicallyIncreasingId(monotonically_increasing_id) => {
                 MonotonicallyIncreasingIdNode::new(
                     self.get_next_pipeline_node_id(),
@@ -268,19 +301,20 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
             )
             .arced(),
             LogicalPlan::Repartition(repartition) => {
-                let RepartitionSpec::Hash(repart_spec) = &repartition.repartition_spec else {
-                    todo!("FLOTILLA_MS3: Support other types of repartition");
-                };
-
-                let columns = BoundExpr::bind_all(&repart_spec.by, &repartition.input.schema())?;
-
-                assert!(!columns.is_empty());
+                match &repartition.repartition_spec {
+                    RepartitionSpec::Hash(repart_spec) => {
+                        assert!(!repart_spec.by.is_empty());
+                    }
+                    RepartitionSpec::Random(_) => {}
+                    RepartitionSpec::IntoPartitions(_) => {
+                        todo!("FLOTILLA_MS3: Support other types of repartition");
+                    }
+                }
                 RepartitionNode::new(
                     self.get_next_pipeline_node_id(),
                     logical_node_id,
                     &self.stage_config,
-                    columns,
-                    repart_spec.num_partitions,
+                    repartition.repartition_spec.clone(),
                     node.schema(),
                     self.curr_node.pop().unwrap(),
                 )
@@ -334,8 +368,12 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     self.get_next_pipeline_node_id(),
                     logical_node_id,
                     &self.stage_config,
-                    columns.clone(),
-                    None,
+                    RepartitionSpec::Hash(
+                        daft_logical_plan::partitioning::HashRepartitionConfig::new(
+                            None,
+                            columns.clone().into_iter().map(|e| e.into()).collect(),
+                        ),
+                    ),
                     distinct.input.schema(),
                     initial_distinct,
                 )
@@ -363,7 +401,7 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                 // First stage: Shuffle by the partition_by columns to colocate rows
                 let input_node = self.curr_node.pop().unwrap();
                 let repartition =
-                    self.gen_shuffle_node(logical_node_id, input_node, partition_by.clone());
+                    self.gen_shuffle_node(logical_node_id, input_node, partition_by.clone(), None);
 
                 // Final stage: The actual window op
                 WindowNode::new(
@@ -380,28 +418,16 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     window.aliases.clone(),
                     window.schema.clone(),
                     repartition,
-                )
+                )?
                 .arced()
             }
             LogicalPlan::Join(join) => {
-                let (remaining_on, _, _, _) = join.on.split_eq_preds();
-                if !remaining_on.is_empty() {
-                    todo!("FLOTILLA_MS?: Implement non-equality joins")
-                }
-
                 // Visitor appends in in-order
                 // TODO: Just use regular recursion?
                 let right_node = self.curr_node.pop().unwrap();
                 let left_node = self.curr_node.pop().unwrap();
 
-                self.gen_hash_join_nodes(
-                    logical_node_id,
-                    join.on.clone(),
-                    left_node,
-                    right_node,
-                    join.join_type,
-                    join.output_schema.clone(),
-                )?
+                self.translate_join(logical_node_id, join, left_node, right_node)?
             }
             LogicalPlan::Sort(sort) => {
                 let sort_by = BoundExpr::bind_all(&sort.sort_by, &sort.input.schema())?;
@@ -434,7 +460,8 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     sort_by.clone(),
                     top_n.descending.clone(),
                     top_n.nulls_first.clone(),
-                    top_n.limit,
+                    top_n.limit + top_n.offset.unwrap_or(0),
+                    Some(0),
                     top_n.input.schema(),
                     self.curr_node.pop().unwrap(),
                 )
@@ -452,6 +479,7 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
                     top_n.descending.clone(),
                     top_n.nulls_first.clone(),
                     top_n.limit,
+                    top_n.offset,
                     top_n.input.schema(),
                     gather,
                 )
@@ -463,8 +491,12 @@ impl TreeNodeVisitor for LogicalPlanToPipelineNodeTranslator {
             LogicalPlan::SubqueryAlias(_)
             | LogicalPlan::Union(_)
             | LogicalPlan::Intersect(_)
-            | LogicalPlan::Shard(_) => {
-                panic!("Logical plan operators SubqueryAlias, Union, Intersect, and Shard should be handled by the optimizer")
+            | LogicalPlan::Shard(_)
+            | LogicalPlan::Offset(_) => {
+                panic!(
+                    "Logical plan operator {} should be handled by the optimizer",
+                    node.name()
+                )
             }
         };
         self.curr_node.push(output);
