@@ -2,6 +2,8 @@
 
 pub use crate::scheduling::{
     scheduler::test_utils::setup_workers,
-    task::tests::{create_mock_partition_ref, MockTask, MockTaskBuilder, MockTaskResultHandle},
+    task::tests::{
+        create_mock_partition_ref, MockTask, MockTaskBuilder, MockTaskFailure, MockTaskResultHandle,
+    },
     worker::tests::MockWorkerManager,
 };
