@@ -36,29 +36,32 @@ impl StagePlanBuilder {
             LogicalPlan::Source(_)
             | LogicalPlan::Project(_)
             | LogicalPlan::Filter(_)
+            | LogicalPlan::IntoBatches(_)
             | LogicalPlan::Sink(_)
             | LogicalPlan::Sample(_)
             | LogicalPlan::Explode(_)
-            | LogicalPlan::ActorPoolProject(_)
+            | LogicalPlan::UDFProject(_)
             | LogicalPlan::Unpivot(_)
             | LogicalPlan::MonotonicallyIncreasingId(_)
             | LogicalPlan::Distinct(_)
             | LogicalPlan::Aggregate(_)
             | LogicalPlan::Window(_)
-            | LogicalPlan::TopN(_)
+            | LogicalPlan::Concat(_)
             | LogicalPlan::Limit(_)
-            | LogicalPlan::Concat(_) => Ok(TreeNodeRecursion::Continue),
-            LogicalPlan::Repartition(repartition) => {
-                if matches!(repartition.repartition_spec, RepartitionSpec::Hash(_)) {
-                    Ok(TreeNodeRecursion::Continue)
-                } else {
+            | LogicalPlan::TopN(_) => Ok(TreeNodeRecursion::Continue),
+            LogicalPlan::Repartition(repartition) => match &repartition.repartition_spec {
+                RepartitionSpec::Hash(_) => Ok(TreeNodeRecursion::Continue),
+                RepartitionSpec::Random(_) => Ok(TreeNodeRecursion::Continue),
+                RepartitionSpec::IntoPartitions(_) => {
                     can_translate = false;
                     Ok(TreeNodeRecursion::Stop)
                 }
             },
             LogicalPlan::Join(join) => {
-                // TODO: Support broadcast join
-                if join.join_strategy.is_some_and(|x| x != JoinStrategy::Hash) {
+                if join
+                    .join_strategy
+                    .is_some_and(|x| !matches!(x, JoinStrategy::Hash | JoinStrategy::Broadcast))
+                {
                     can_translate = false;
                     Ok(TreeNodeRecursion::Stop)
                 } else {
@@ -89,7 +92,11 @@ impl StagePlanBuilder {
             LogicalPlan::Intersect(_)
             | LogicalPlan::Union(_)
             | LogicalPlan::SubqueryAlias(_)
-            | LogicalPlan::Shard(_) => panic!("Intersect, Union, SubqueryAlias, and Shard should be optimized away before planning stages")
+            | LogicalPlan::Shard(_)
+            | LogicalPlan::Offset(_) => panic!(
+                "Logical plan operator {} should be optimized away before planning stages",
+                node.name()
+            ),
         });
         can_translate
     }
