@@ -24,10 +24,12 @@ use daft_core::{
         try_stddev_aggregation_supertype, try_sum_supertype, InferDataType,
     },
     join::JoinSide,
+    lit::Literal,
     prelude::*,
     utils::supertype::{try_get_collection_supertype, try_get_supertype},
 };
 use derive_more::Display;
+use itertools::Itertools;
 use serde::{Deserialize, Serialize};
 
 use super::functions::FunctionExpr;
@@ -35,14 +37,14 @@ use crate::{
     expr::bound_expr::BoundExpr,
     functions::{
         function_display_without_formatter, function_semantic_id,
-        python::PythonUDF,
-        scalar::scalar_function_semantic_id,
+        python::LegacyPythonUDF,
+        scalar::{scalar_function_semantic_id, ScalarFn},
         sketch::{HashableVecPercentiles, SketchExpr},
         struct_::StructExpr,
-        FunctionArg, FunctionArgs, FunctionEvaluator, ScalarFunction, FUNCTION_REGISTRY,
+        BuiltinScalarFn, FunctionArg, FunctionArgs, FunctionEvaluator, FUNCTION_REGISTRY,
     },
-    lit,
     optimization::{get_required_columns, requires_computation},
+    python_udf::{PyScalarFn, RowWisePyFn},
 };
 
 pub trait SubqueryPlan: std::fmt::Debug + std::fmt::Display + Send + Sync {
@@ -274,7 +276,7 @@ pub enum Expr {
     List(Vec<ExprRef>),
 
     #[display("lit({_0})")]
-    Literal(lit::LiteralValue),
+    Literal(Literal),
 
     #[display("if [{predicate}] then [{if_true}] else [{if_false}]")]
     IfElse {
@@ -284,7 +286,7 @@ pub enum Expr {
     },
 
     #[display("{_0}")]
-    ScalarFunction(ScalarFunction),
+    ScalarFn(ScalarFn),
 
     #[display("subquery {_0}")]
     Subquery(Subquery),
@@ -398,6 +400,14 @@ pub enum SketchType {
     HyperLogLog,
 }
 
+pub fn lit<L: Into<Literal>>(t: L) -> ExprRef {
+    Arc::new(Expr::Literal(t.into()))
+}
+
+pub fn null_lit() -> ExprRef {
+    Arc::new(Expr::Literal(Literal::Null))
+}
+
 /// Unresolved column with no associated plan ID or schema.
 pub fn unresolved_col(name: impl Into<Arc<str>>) -> ExprRef {
     UnresolvedColumn {
@@ -432,6 +442,30 @@ pub fn binary_op(op: Operator, left: ExprRef, right: ExprRef) -> ExprRef {
 }
 
 impl AggExpr {
+    pub fn agg_name(&self) -> &'static str {
+        match self {
+            Self::Count(_, _) => "Count",
+            Self::CountDistinct(_) => "Count Distinct",
+            Self::Sum(_) => "Sum",
+            Self::ApproxPercentile(_) => "Approx Percentile",
+            Self::ApproxCountDistinct(_) => "Approx Count Distinct",
+            Self::ApproxSketch(_, _) => "Approx Sketch",
+            Self::MergeSketch(_, _) => "Merge Sketch",
+            Self::Mean(_) => "Mean",
+            Self::Stddev(_) => "Stddev",
+            Self::Min(_) => "Min",
+            Self::Max(_) => "Max",
+            Self::BoolAnd(_) => "Bool And",
+            Self::BoolOr(_) => "Bool Or",
+            Self::AnyValue(_, _) => "Any Value",
+            Self::List(_) => "List",
+            Self::Set(_) => "Set",
+            Self::Concat(_) => "Concat",
+            Self::Skew(_) => "Skew",
+            Self::MapGroups { .. } => "Map Groups",
+        }
+    }
+
     pub fn name(&self) -> &str {
         match self {
             Self::Count(expr, ..)
@@ -706,7 +740,7 @@ impl AggExpr {
                 Ok(Field::new(field.name.as_str(), field.dtype))
             }
 
-            Self::List(expr) | Self::Set(expr) => expr.to_field(schema)?.to_list_field(),
+            Self::List(expr) | Self::Set(expr) => Ok(expr.to_field(schema)?.to_list_field()),
 
             Self::BoolAnd(expr) | Self::BoolOr(expr) => {
                 let field = expr.to_field(schema)?;
@@ -1211,7 +1245,7 @@ impl Expr {
             Self::Alias(expr, ..) => expr.semantic_id(schema),
             // Agg: Separate path.
             Self::Agg(agg_expr) => agg_expr.semantic_id(schema),
-            Self::ScalarFunction(sf) => scalar_function_semantic_id(sf, schema),
+            Self::ScalarFn(ScalarFn::Builtin(sf)) => scalar_function_semantic_id(sf, schema),
             Self::Subquery(subquery) => subquery.semantic_id(),
             Self::InSubquery(expr, subquery) => {
                 let child_id = expr.semantic_id(schema);
@@ -1261,6 +1295,17 @@ impl Expr {
                 let child_id = window_expr.semantic_id(schema);
                 FieldID::new(format!("{child_id}.window_function()"))
             }
+            Self::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                function_name: name,
+                args: children,
+                ..
+            }))) => {
+                let children_ids = children
+                    .iter()
+                    .map(|expr| expr.semantic_id(schema).id)
+                    .join(",");
+                FieldID::new(format!("ScalarPythonUDF_{name}({children_ids})"))
+            }
         }
     }
 
@@ -1300,7 +1345,11 @@ impl Expr {
                 vec![if_true.clone(), if_false.clone(), predicate.clone()]
             }
             Self::FillNull(expr, fill_value) => vec![expr.clone(), fill_value.clone()],
-            Self::ScalarFunction(sf) => sf.inputs.clone().into_inner(),
+            Self::ScalarFn(ScalarFn::Builtin(sf)) => sf.inputs.clone().into_inner(),
+            Self::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                args: children,
+                ..
+            }))) => children.clone(),
         }
     }
 
@@ -1395,7 +1444,7 @@ impl Expr {
                     inputs: children,
                 }
             }
-            Self::ScalarFunction(sf) => {
+            Self::ScalarFn(ScalarFn::Builtin(sf)) => {
                 assert!(
                     children.len() == sf.inputs.len(),
                     "Should have same number of children"
@@ -1413,10 +1462,30 @@ impl Expr {
                     })
                     .collect();
 
-                Self::ScalarFunction(crate::functions::ScalarFunction {
+                Self::ScalarFn(ScalarFn::Builtin(BuiltinScalarFn {
                     udf: sf.udf.clone(),
                     inputs: FunctionArgs::new_unchecked(new_children),
-                })
+                }))
+            }
+            Self::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                function_name: name,
+                inner: func,
+                return_dtype,
+                original_args,
+                args: old_children,
+            }))) => {
+                assert!(
+                    children.len() == old_children.len(),
+                    "Should have same number of children"
+                );
+
+                Self::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                    function_name: name.clone(),
+                    inner: func.clone(),
+                    return_dtype: return_dtype.clone(),
+                    original_args: original_args.clone(),
+                    args: children,
+                })))
             }
         }
     }
@@ -1506,7 +1575,7 @@ impl Expr {
             }
             Self::Literal(value) => Ok(Field::new("literal", value.get_type())),
             Self::Function { func, inputs } => func.to_field(inputs.as_slice(), schema, func),
-            Self::ScalarFunction(sf) => sf.to_field(schema),
+            Self::ScalarFn(sf) => sf.to_field(schema),
             Self::BinaryOp { op, left, right } => {
                 let left_field = left.to_field(schema)?;
                 let right_field = right.to_field(schema)?;
@@ -1588,8 +1657,8 @@ impl Expr {
                     )));
                 }
                 match predicate.as_ref() {
-                    Self::Literal(lit::LiteralValue::Boolean(true)) => if_true.to_field(schema),
-                    Self::Literal(lit::LiteralValue::Boolean(false)) => {
+                    Self::Literal(Literal::Boolean(true)) => if_true.to_field(schema),
+                    Self::Literal(Literal::Boolean(false)) => {
                         Ok(if_false.to_field(schema)?.rename(if_true.name()))
                     }
                     _ => {
@@ -1651,7 +1720,7 @@ impl Expr {
                 FunctionExpr::Struct(StructExpr::Get(name)) => name,
                 _ => inputs.first().unwrap().name(),
             },
-            Self::ScalarFunction(func) => match func.name() {
+            Self::ScalarFn(ScalarFn::Builtin(func)) => match func.name() {
                 "struct" => "struct", // FIXME: make struct its own expr variant
                 "monotonically_increasing_id" => "monotonically_increasing_id", // Special case for functions with no inputs
                 _ => func.inputs.first().unwrap().name(),
@@ -1667,6 +1736,17 @@ impl Expr {
             Self::Exists(subquery) => subquery.name(),
             Self::Over(expr, ..) => expr.name(),
             Self::WindowFunction(expr) => expr.name(),
+            Self::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
+                function_name: name,
+                args: children,
+                ..
+            }))) => {
+                if let Some(first_child) = children.first() {
+                    first_child.name()
+                } else {
+                    name.as_ref()
+                }
+            }
         }
     }
 
@@ -1747,7 +1827,7 @@ impl Expr {
                 | Expr::Between(..)
                 | Expr::Function { .. }
                 | Expr::FillNull(..)
-                | Expr::ScalarFunction { .. }
+                | Expr::ScalarFn { .. }
                 | Expr::Subquery(..)
                 | Expr::InSubquery(..)
                 | Expr::Exists(..)
@@ -1766,7 +1846,7 @@ impl Expr {
     }
 
     /// Returns the literal value if this is a literal expression, otherwise none.
-    pub fn as_literal(&self) -> Option<&lit::LiteralValue> {
+    pub fn as_literal(&self) -> Option<&Literal> {
         match self {
             Self::Literal(lit) => Some(lit),
             _ => None,
@@ -1788,7 +1868,7 @@ impl Expr {
             Self::Subquery(..) => false,
             Self::Exists(..) => false,
             Self::Function { .. } => true,
-            Self::ScalarFunction(..) => true,
+            Self::ScalarFn(..) => true,
             Self::Agg(_) => true,
             Self::Over(..) => true,
             Self::WindowFunction(..) => true,
@@ -1841,14 +1921,22 @@ impl Expr {
             .data)
     }
 
+    pub fn unwrap_alias(self: &ExprRef) -> (ExprRef, Option<Arc<str>>) {
+        match self.as_ref() {
+            // Recursively unwrap if nested aliases, but only return the outermost
+            Self::Alias(expr, name) => (expr.clone().unwrap_alias().0, Some(name.clone())),
+            _ => (self.clone(), None),
+        }
+    }
+
     pub fn explode(self: Arc<Self>) -> DaftResult<ExprRef> {
         let explode_fn = FUNCTION_REGISTRY.read().unwrap().get("explode").unwrap();
         let f = explode_fn.get_function(FunctionArgs::empty(), &Schema::empty())?;
 
-        Ok(Self::ScalarFunction(ScalarFunction {
+        Ok(Self::ScalarFn(ScalarFn::Builtin(BuiltinScalarFn {
             udf: f,
             inputs: FunctionArgs::new_unchecked(vec![FunctionArg::Unnamed(self)]),
-        })
+        }))
         .arced())
     }
 }
@@ -1943,7 +2031,11 @@ impl FromStr for Operator {
 }
 
 // Check if one set of columns is a reordering of the other
-pub fn is_partition_compatible(a: &[ExprRef], b: &[ExprRef]) -> bool {
+pub fn is_partition_compatible<'a, A, B>(a: A, b: B) -> bool
+where
+    A: IntoIterator<Item = &'a ExprRef>,
+    B: IntoIterator<Item = &'a ExprRef>,
+{
     // sort a and b by name
     let a_set: HashSet<&ExprRef> = HashSet::from_iter(a);
     let b_set: HashSet<&ExprRef> = HashSet::from_iter(b);
@@ -1972,7 +2064,7 @@ pub fn is_actor_pool_udf(expr: &ExprRef) -> bool {
     matches!(
         expr.as_ref(),
         Expr::Function {
-            func: FunctionExpr::Python(PythonUDF {
+            func: FunctionExpr::Python(LegacyPythonUDF {
                 concurrency: Some(_),
                 ..
             }),
@@ -1981,6 +2073,36 @@ pub fn is_actor_pool_udf(expr: &ExprRef) -> bool {
     )
 }
 
+#[inline]
+pub fn is_udf(expr: &ExprRef) -> bool {
+    matches!(
+        expr.as_ref(),
+        Expr::Function {
+            func: FunctionExpr::Python(LegacyPythonUDF { .. }),
+            ..
+        } | Expr::ScalarFn(ScalarFn::Python(_))
+    )
+}
+
+pub fn count_udfs(exprs: &[ExprRef]) -> usize {
+    exprs
+        .iter()
+        .map(|expr| {
+            let mut count = 0;
+            expr.apply(|e| {
+                if is_udf(e) {
+                    count += 1;
+                }
+
+                Ok(common_treenode::TreeNodeRecursion::Continue)
+            })
+            .unwrap();
+
+            count
+        })
+        .sum()
+}
+
 pub fn count_actor_pool_udfs(exprs: &[ExprRef]) -> usize {
     exprs
         .iter()
@@ -2054,16 +2176,20 @@ pub fn estimated_selectivity(expr: &Expr, schema: &Schema) -> f64 {
 
         // Boolean literals
         Expr::Literal(lit) => match lit {
-            lit::LiteralValue::Boolean(true) => 1.0,
-            lit::LiteralValue::Boolean(false) => 0.0,
+            Literal::Boolean(true) => 1.0,
+            Literal::Boolean(false) => 0.0,
             _ => 1.0,
         },
 
         // String contains
-        Expr::ScalarFunction(ScalarFunction { udf, .. }) if udf.name() == "contains" => 0.1,
+        Expr::ScalarFn(ScalarFn::Builtin(BuiltinScalarFn { udf, .. }))
+            if udf.name() == "contains" =>
+        {
+            0.1
+        }
 
         // Everything else that could be boolean gets 0.2, non-boolean gets 1.0
-        Expr::ScalarFunction(_)
+        Expr::ScalarFn(_)
         | Expr::Function { .. }
         | Expr::Column(_)
         | Expr::IfElse { .. }
