@@ -12,23 +12,19 @@ use daft_core::{
     datatypes::{IntervalValue, IntervalValueBuilder},
     prelude::*,
     python::{PyDataType, PyField, PySchema, PySeries, PyTimeUnit},
-    utils::display::display_decimal128,
 };
-use indexmap::IndexMap;
 use pyo3::{
     exceptions::PyValueError,
-    intern,
     prelude::*,
     pyclass::CompareOp,
-    types::{PyBool, PyBytes, PyFloat, PyInt, PyNone, PyString},
-    IntoPyObjectExt,
+    types::{PyBool, PyBytes, PyFloat, PyInt, PyString},
 };
 use serde::{Deserialize, Serialize};
 
 use crate::{
     expr::{Expr, WindowExpr},
     visitor::accept,
-    ExprRef, LiteralValue, Operator,
+    ExprRef, Operator,
 };
 
 #[pyfunction]
@@ -41,27 +37,32 @@ pub fn resolved_col(name: &str) -> PyExpr {
     PyExpr::from(crate::resolved_col(name))
 }
 
+#[pyfunction]
+pub fn bound_col(index: usize, field: PyField) -> PyExpr {
+    crate::bound_col(index, field.field).into()
+}
+
 #[pyfunction]
 pub fn date_lit(item: i32) -> PyResult<PyExpr> {
-    let expr = Expr::Literal(LiteralValue::Date(item));
+    let expr = Expr::Literal(Literal::Date(item));
     Ok(expr.into())
 }
 
 #[pyfunction]
 pub fn time_lit(item: i64, tu: PyTimeUnit) -> PyResult<PyExpr> {
-    let expr = Expr::Literal(LiteralValue::Time(item, tu.timeunit));
+    let expr = Expr::Literal(Literal::Time(item, tu.timeunit));
     Ok(expr.into())
 }
 
 #[pyfunction(signature = (val, tu, tz=None))]
 pub fn timestamp_lit(val: i64, tu: PyTimeUnit, tz: Option<String>) -> PyResult<PyExpr> {
-    let expr = Expr::Literal(LiteralValue::Timestamp(val, tu.timeunit, tz));
+    let expr = Expr::Literal(Literal::Timestamp(val, tu.timeunit, tz));
     Ok(expr.into())
 }
 
 #[pyfunction]
 pub fn duration_lit(val: i64, tu: PyTimeUnit) -> PyResult<PyExpr> {
-    let expr = Expr::Literal(LiteralValue::Duration(val, tu.timeunit));
+    let expr = Expr::Literal(Literal::Duration(val, tu.timeunit));
     Ok(expr.into())
 }
 
@@ -115,7 +116,7 @@ pub fn interval_lit(
         nanoseconds: nanos,
     };
     let iv = IntervalValue::try_new(opts)?;
-    let expr = Expr::Literal(LiteralValue::Interval(iv));
+    let expr = Expr::Literal(Literal::Interval(iv));
     Ok(expr.into())
 }
 
@@ -160,7 +161,7 @@ pub fn decimal_lit(sign: bool, digits: Vec<u8>, exp: i32) -> PyResult<PyExpr> {
     if sign {
         v = -v;
     }
-    let expr = Expr::Literal(LiteralValue::Decimal(
+    let expr = Expr::Literal(Literal::Decimal(
         v,
         u8::try_from(precision)?,
         i8::try_from(scale)?,
@@ -169,8 +170,8 @@ pub fn decimal_lit(sign: bool, digits: Vec<u8>, exp: i32) -> PyResult<PyExpr> {
 }
 
 #[pyfunction]
-pub fn series_lit(series: PySeries) -> PyResult<PyExpr> {
-    let expr = Expr::Literal(LiteralValue::Series(series.series));
+pub fn list_lit(series: PySeries) -> PyResult<PyExpr> {
+    let expr = Expr::Literal(Literal::List(series.series));
     Ok(expr.into())
 }
 
@@ -179,38 +180,39 @@ pub fn lit(item: Bound<PyAny>) -> PyResult<PyExpr> {
     literal_value(item).map(Expr::Literal).map(Into::into)
 }
 
-pub fn literal_value(item: Bound<PyAny>) -> PyResult<LiteralValue> {
+pub fn literal_value(item: Bound<PyAny>) -> PyResult<Literal> {
     if item.is_instance_of::<PyBool>() {
         let val = item.extract::<bool>()?;
-        Ok(crate::literal_value(val))
+        Ok(val.into())
     } else if let Ok(int) = item.downcast::<PyInt>() {
         match int.extract::<i64>() {
             Ok(val) => {
                 if val >= 0 && val < i32::MAX as i64 || val <= 0 && val > i32::MIN as i64 {
-                    Ok(crate::literal_value(val as i32))
+                    Ok((val as i32).into())
                 } else {
-                    Ok(crate::literal_value(val))
+                    Ok(val.into())
                 }
             }
             _ => {
                 let val = int.extract::<u64>()?;
-                Ok(crate::literal_value(val))
+                Ok(val.into())
             }
         }
     } else if let Ok(float) = item.downcast::<PyFloat>() {
         let val = float.extract::<f64>()?;
-        Ok(crate::literal_value(val))
+        Ok(val.into())
     } else if let Ok(pystr) = item.downcast::<PyString>() {
-        Ok(crate::literal_value(pystr.extract::<String>().expect(
-            "could not transform Python string to Rust Unicode",
-        )))
+        Ok(pystr
+            .extract::<String>()
+            .expect("could not transform Python string to Rust Unicode")
+            .into())
     } else if let Ok(pybytes) = item.downcast::<PyBytes>() {
         let bytes = pybytes.as_bytes();
-        Ok(crate::literal_value(bytes))
+        Ok(bytes.into())
     } else if item.is_none() {
-        Ok(LiteralValue::Null)
+        Ok(Literal::Null)
     } else {
-        Ok(crate::literal_value::<PyObject>(item.into()))
+        Ok(PyObject::from(item).into())
     }
 }
 
@@ -229,7 +231,8 @@ pub fn list_(items: Vec<PyExpr>) -> PyExpr {
     init_args,
     resource_request=None,
     batch_size=None,
-    concurrency=None
+    concurrency=None,
+    use_process=None
 ))]
 pub fn udf(
     name: &str,
@@ -241,6 +244,7 @@ pub fn udf(
     resource_request: Option<ResourceRequest>,
     batch_size: Option<usize>,
     concurrency: Option<usize>,
+    use_process: Option<bool>,
 ) -> PyResult<PyExpr> {
     use crate::functions::python::udf;
 
@@ -264,11 +268,36 @@ pub fn udf(
             resource_request,
             batch_size,
             concurrency,
+            use_process,
         )?
         .into(),
     })
 }
 
+#[pyfunction]
+pub fn row_wise_udf(
+    name: &str,
+    inner: PyObject,
+    return_dtype: PyDataType,
+    original_args: PyObject,
+    expr_args: Vec<PyExpr>,
+) -> PyExpr {
+    use crate::python_udf::row_wise_udf;
+
+    let args = expr_args.into_iter().map(|pyexpr| pyexpr.expr).collect();
+
+    PyExpr {
+        expr: row_wise_udf(
+            name,
+            inner.into(),
+            return_dtype.into(),
+            original_args.into(),
+            args,
+        )
+        .into(),
+    }
+}
+
 /// Initializes all uninitialized UDFs in the expression
 #[pyfunction]
 pub fn initialize_udfs(expr: PyExpr) -> PyResult<PyExpr> {
@@ -278,9 +307,9 @@ pub fn initialize_udfs(expr: PyExpr) -> PyResult<PyExpr> {
 
 /// Get the names of all UDFs in expression
 #[pyfunction]
-pub fn get_udf_names(expr: PyExpr) -> Vec<String> {
-    use crate::functions::python::get_udf_names;
-    get_udf_names(&expr.expr)
+pub fn try_get_udf_name(expr: PyExpr) -> Option<String> {
+    use crate::functions::python::try_get_udf_name;
+    try_get_udf_name(&expr.expr)
 }
 
 #[pyclass(module = "daft.daft")]
@@ -644,125 +673,3 @@ impl From<&PyExpr> for crate::ExprRef {
         item.expr.clone()
     }
 }
-
-impl<'py> IntoPyObject<'py> for LiteralValue {
-    type Target = PyAny;
-
-    type Output = Bound<'py, Self::Target>;
-
-    type Error = PyErr;
-
-    fn into_pyobject(self, py: Python<'py>) -> Result<Self::Output, Self::Error> {
-        fn div_rem(l: i64, r: i64) -> (i64, i64) {
-            (l / r, l % r)
-        }
-
-        match self {
-            Self::Null => Ok(PyNone::get(py).to_owned().into_any()),
-            Self::Boolean(val) => val.into_bound_py_any(py),
-            Self::Utf8(val) => val.into_bound_py_any(py),
-            Self::Binary(val) => val.into_bound_py_any(py),
-            Self::FixedSizeBinary(val, _) => val.into_bound_py_any(py),
-            Self::Int8(val) => val.into_bound_py_any(py),
-            Self::UInt8(val) => val.into_bound_py_any(py),
-            Self::Int16(val) => val.into_bound_py_any(py),
-            Self::UInt16(val) => val.into_bound_py_any(py),
-            Self::Int32(val) => val.into_bound_py_any(py),
-            Self::UInt32(val) => val.into_bound_py_any(py),
-            Self::Int64(val) => val.into_bound_py_any(py),
-            Self::UInt64(val) => val.into_bound_py_any(py),
-            Self::Timestamp(val, time_unit, tz) => {
-                let ts = (val as f64) / (time_unit.to_scale_factor() as f64);
-
-                py.import(intern!(py, "datetime"))?
-                    .getattr(intern!(py, "datetime"))?
-                    .call_method1(intern!(py, "fromtimestamp"), (ts, tz))
-            }
-            Self::Date(val) => py
-                .import(intern!(py, "datetime"))?
-                .getattr(intern!(py, "date"))?
-                .call_method1(intern!(py, "fromtimestamp"), (val,)),
-            Self::Time(val, time_unit) => {
-                let (h, m, s, us) = match time_unit {
-                    TimeUnit::Nanoseconds => {
-                        let (h, rem) = div_rem(val, 60 * 60 * 1_000_000_000);
-                        let (m, rem) = div_rem(rem, 60 * 1_000_000_000);
-                        let (s, rem) = div_rem(rem, 1_000_000_000);
-                        let us = rem / 1_000;
-                        (h, m, s, us)
-                    }
-                    TimeUnit::Microseconds => {
-                        let (h, rem) = div_rem(val, 60 * 60 * 1_000_000);
-                        let (m, rem) = div_rem(rem, 60 * 1_000_000);
-                        let (s, us) = div_rem(rem, 1_000_000);
-                        (h, m, s, us)
-                    }
-                    TimeUnit::Milliseconds => {
-                        let (h, rem) = div_rem(val, 60 * 60 * 1_000);
-                        let (m, rem) = div_rem(rem, 60 * 1_000);
-                        let (s, ms) = div_rem(rem, 1_000);
-                        let us = ms * 1_000;
-                        (h, m, s, us)
-                    }
-                    TimeUnit::Seconds => {
-                        let (h, rem) = div_rem(val, 60 * 60);
-                        let (m, s) = div_rem(rem, 60);
-                        (h, m, s, 0)
-                    }
-                };
-
-                py.import(intern!(py, "datetime"))?
-                    .getattr(intern!(py, "time"))?
-                    .call1((h, m, s, us))
-            }
-            Self::Duration(val, time_unit) => {
-                let (d, s, us) = match time_unit {
-                    TimeUnit::Nanoseconds => {
-                        let (d, rem) = div_rem(val, 24 * 60 * 60 * 1_000_000_000);
-                        let (s, rem) = div_rem(rem, 1_000_000_000);
-                        let us = rem / 1_000;
-                        (d, s, us)
-                    }
-                    TimeUnit::Microseconds => {
-                        let (d, rem) = div_rem(val, 24 * 60 * 60 * 1_000_000);
-                        let (s, us) = div_rem(rem, 1_000_000);
-                        (d, s, us)
-                    }
-                    TimeUnit::Milliseconds => {
-                        let (d, rem) = div_rem(val, 24 * 60 * 60 * 1_000);
-                        let (s, ms) = div_rem(rem, 1_000);
-                        let us = ms * 1_000;
-                        (d, s, us)
-                    }
-                    TimeUnit::Seconds => {
-                        let (d, s) = div_rem(val, 24 * 60 * 60);
-                        (d, s, 0)
-                    }
-                };
-
-                py.import(intern!(py, "datetime"))?
-                    .getattr(intern!(py, "timedelta"))?
-                    .call1((d, s, us))
-            }
-            Self::Interval(_) => {
-                Err(DaftError::NotImplemented("Interval literal to Python".to_string()).into())
-            }
-            Self::Float64(val) => val.into_bound_py_any(py),
-            Self::Decimal(val, p, s) => py
-                .import(intern!(py, "decimal"))?
-                .getattr(intern!(py, "Decimal"))?
-                .call1((display_decimal128(val, p, s),)),
-            Self::Series(series) => py
-                .import(intern!(py, "daft.series"))?
-                .getattr(intern!(py, "Series"))?
-                .getattr(intern!(py, "_from_pyseries"))?
-                .call1((PySeries { series },)),
-            Self::Python(val) => val.0.as_ref().into_bound_py_any(py),
-            Self::Struct(entries) => entries
-                .into_iter()
-                .map(|(f, v)| (f.name, v))
-                .collect::<IndexMap<_, _>>()
-                .into_bound_py_any(py),
-        }
-    }
-}
