@@ -1,8 +1,8 @@
 use common_error::{ensure, DaftResult};
 use daft_core::prelude::*;
 use daft_dsl::{
-    functions::{FunctionArgs, ScalarFunction, ScalarUDF},
-    lit, literals_to_series, ExprRef, Literal,
+    functions::{scalar::ScalarFn, FunctionArgs, ScalarUDF},
+    lit, ExprRef,
 };
 use serde::{Deserialize, Serialize};
 
@@ -23,7 +23,7 @@ impl ScalarUDF for ListSort {
         let desc = inputs
             .optional((1, "desc"))?
             .cloned()
-            .unwrap_or_else(|| literals_to_series(&[false.literal_value()]).unwrap());
+            .unwrap_or_else(|| Literal::from(false).into());
 
         let nulls_first = inputs
             .optional((2, "nulls_first"))?
@@ -64,5 +64,5 @@ impl ScalarUDF for ListSort {
 pub fn list_sort(input: ExprRef, desc: Option<ExprRef>, nulls_first: Option<ExprRef>) -> ExprRef {
     let desc = desc.unwrap_or_else(|| lit(false));
     let nulls_first = nulls_first.unwrap_or_else(|| desc.clone());
-    ScalarFunction::new(ListSort {}, vec![input, desc, nulls_first]).into()
+    ScalarFn::builtin(ListSort {}, vec![input, desc, nulls_first]).into()
 }
