@@ -4,7 +4,6 @@ use std::{
     collections::HashMap,
     io::Write,
     num::{NonZeroI32, NonZeroUsize},
-    ops::Range,
     pin::Pin,
     string::FromUtf8Error,
     sync::Arc,
@@ -51,11 +50,12 @@ use url::{ParseError, Position};
 use super::object_io::{GetResult, ObjectSource};
 use crate::{
     object_io::{FileMetadata, FileType, LSResult},
+    range::GetRange,
     retry::{ExponentialBackoff, RetryError},
     stats::IOStatsRef,
     stream_utils::io_stats_on_bytestream,
     Error::InvalidArgument,
-    FileFormat, InvalidArgumentSnafu, SourceType,
+    FileFormat, InvalidArgumentSnafu, InvalidRangeRequestSnafu, SourceType,
 };
 
 const S3_DELIMITER: &str = "/";
@@ -67,7 +67,6 @@ pub struct S3LikeSource {
     connection_pool_sema: Arc<tokio::sync::Semaphore>,
     default_region: Region,
     s3_config: S3Config,
-    anonymous: bool,
 }
 
 #[derive(Debug, Snafu)]
@@ -192,9 +191,6 @@ enum Error {
         "Unable to parse data as Utf8 while reading header for file: {path}. {source}"
     ))]
     UnableToParseUtf8 { path: String, source: FromUtf8Error },
-
-    #[snafu(display("Uploads cannot be anonymous. Please disable anonymous S3 access."))]
-    UploadsCannotBeAnonymous {},
 }
 
 /// List of AWS error codes that are due to throttling
@@ -480,7 +476,7 @@ async fn provide_credentials_with_retry(
     Ok(creds)
 }
 
-async fn build_s3_conf(config: &S3Config) -> super::Result<(bool, s3::Config)> {
+async fn build_s3_conf(config: &S3Config) -> super::Result<s3::Config> {
     const DEFAULT_REGION: Region = Region::from_static("us-east-1");
 
     let region = if let Some(region_name) = &config.region_name {
@@ -528,8 +524,6 @@ async fn build_s3_conf(config: &S3Config) -> super::Result<(bool, s3::Config)> {
         }
     };
 
-    let anonymous = credentials_provider.is_none();
-
     let identity_cache = config.buffer_time.map(|buffer_time| {
         IdentityCache::lazy()
             .buffer_time(Duration::from_secs(buffer_time))
@@ -631,16 +625,16 @@ async fn build_s3_conf(config: &S3Config) -> super::Result<(bool, s3::Config)> {
 
     let s3_conf = builder.build();
 
-    Ok((anonymous, s3_conf))
+    Ok(s3_conf)
 }
 
-async fn build_s3_client(config: &S3Config) -> super::Result<(bool, s3::Client)> {
-    let (anonymous, s3_conf) = build_s3_conf(config).await?;
-    Ok((anonymous, s3::Client::from_conf(s3_conf)))
+async fn build_s3_client(config: &S3Config) -> super::Result<s3::Client> {
+    let s3_conf = build_s3_conf(config).await?;
+    Ok(s3::Client::from_conf(s3_conf))
 }
 
 async fn build_client(config: &S3Config) -> super::Result<S3LikeSource> {
-    let (anonymous, client) = build_s3_client(config).await?;
+    let client = build_s3_client(config).await?;
     let mut client_map = HashMap::new();
     let default_region = client.config().region().unwrap().clone();
     client_map.insert(default_region.clone(), client.into());
@@ -652,7 +646,6 @@ async fn build_client(config: &S3Config) -> super::Result<S3LikeSource> {
         )),
         s3_config: config.clone(),
         default_region,
-        anonymous,
     })
 }
 const REGION_HEADER: &str = "x-amz-bucket-region";
@@ -678,7 +671,7 @@ impl S3LikeSource {
         let mut new_config = self.s3_config.clone();
         new_config.region_name = Some(region.to_string());
 
-        let (_, new_client) = build_s3_client(&new_config).await?;
+        let new_client = build_s3_client(&new_config).await?;
 
         if w_handle.get(region).is_none() {
             w_handle.insert(region.clone(), new_client.into());
@@ -691,7 +684,7 @@ impl S3LikeSource {
         &self,
         permit: OwnedSemaphorePermit,
         uri: &str,
-        range: Option<Range<usize>>,
+        range: Option<GetRange>,
         region: &Region,
     ) -> super::Result<GetResult> {
         log::debug!("S3 get at {uri}, range: {range:?}, in region: {region}");
@@ -716,11 +709,10 @@ impl S3LikeSource {
 
             let request = match &range {
                 None => request,
-                Some(range) => request.range(format!(
-                    "bytes={}-{}",
-                    range.start,
-                    range.end.saturating_sub(1)
-                )),
+                Some(range) => {
+                    range.validate().context(InvalidRangeRequestSnafu)?;
+                    request.range(range.to_string())
+                }
             };
 
             let response = request.send().await;
@@ -999,13 +991,7 @@ impl S3LikeSource {
                 request
             };
 
-            let response = if self.anonymous {
-                return Err(Error::UploadsCannotBeAnonymous {}.into());
-            } else {
-                request.send().await
-            };
-
-            match response {
+            match request.send().await {
                 Ok(_) => Ok(()),
                 Err(err) => Err(UnableToPutFileSnafu { path: uri }.into_error(err).into()),
             }
@@ -1018,10 +1004,6 @@ impl S3LikeSource {
         bucket: &str,
         key: &str,
     ) -> super::Result<Cow<'static, str>> {
-        if self.anonymous {
-            return Err(Error::UploadsCannotBeAnonymous {}.into());
-        }
-
         let region = &self.default_region;
 
         let _permit = self
@@ -1068,10 +1050,6 @@ impl S3LikeSource {
         upload_id: Cow<'static, str>,
         completed_parts: Vec<CompletedPart>,
     ) -> super::Result<()> {
-        if self.anonymous {
-            return Err(Error::UploadsCannotBeAnonymous {}.into());
-        }
-
         let region = &self.default_region;
 
         let _permit = self
@@ -1120,10 +1098,6 @@ impl S3LikeSource {
         part_number: NonZeroI32,
         data: bytes::Bytes,
     ) -> super::Result<UploadPartOutput> {
-        if self.anonymous {
-            return Err(Error::UploadsCannotBeAnonymous {}.into());
-        }
-
         let region = &self.default_region;
 
         let _permit = self
@@ -1167,7 +1141,7 @@ impl ObjectSource for S3LikeSource {
     async fn get(
         &self,
         uri: &str,
-        range: Option<Range<usize>>,
+        range: Option<GetRange>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<GetResult> {
         let permit = self
