@@ -15,48 +15,41 @@ use crate::{
         Sender,
     },
     dispatcher::{DispatchSpawner, RoundRobinDispatcher, UnorderedDispatcher},
-    pipeline::{NodeInfo, PipelineNode, RuntimeContext},
-    progress_bar::ProgressBarColor,
+    ops::{NodeCategory, NodeInfo, NodeType},
+    pipeline::{NodeName, PipelineNode, RuntimeContext},
     resource_manager::MemoryManager,
     runtime_stats::{
-        BaseStatsBuilder, CountingReceiver, CountingSender, RuntimeStatsBuilder,
-        RuntimeStatsContext, RuntimeStatsEventHandler,
+        CountingSender, DefaultRuntimeStats, InitializingCountingReceiver, RuntimeStats,
     },
     ExecutionRuntimeContext, ExecutionTaskSpawner, OperatorOutput, PipelineExecutionSnafu,
 };
 
-pub(crate) trait IntermediateOpState: Send + Sync {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any;
-}
-
-struct DefaultIntermediateOperatorState {}
-impl IntermediateOpState for DefaultIntermediateOperatorState {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any {
-        self
-    }
-}
-
 pub enum IntermediateOperatorResult {
     NeedMoreInput(Option<Arc<MicroPartition>>),
     HasMoreOutput(Arc<MicroPartition>),
 }
 
-pub(crate) type IntermediateOpExecuteResult =
-    OperatorOutput<DaftResult<(Box<dyn IntermediateOpState>, IntermediateOperatorResult)>>;
-pub trait IntermediateOperator: Send + Sync {
+pub(crate) type IntermediateOpExecuteResult<Op> = OperatorOutput<
+    DaftResult<(
+        <Op as IntermediateOperator>::State,
+        IntermediateOperatorResult,
+    )>,
+>;
+pub(crate) trait IntermediateOperator: Send + Sync {
+    type State: Send + Sync + Unpin;
+
     fn execute(
         &self,
         input: Arc<MicroPartition>,
-        state: Box<dyn IntermediateOpState>,
+        state: Self::State,
         task_spawner: &ExecutionTaskSpawner,
-    ) -> IntermediateOpExecuteResult;
-    fn name(&self) -> &'static str;
+    ) -> IntermediateOpExecuteResult<Self>;
+    fn name(&self) -> NodeName;
+    fn op_type(&self) -> NodeType;
     fn multiline_display(&self) -> Vec<String>;
-    fn make_state(&self) -> DaftResult<Box<dyn IntermediateOpState>> {
-        Ok(Box::new(DefaultIntermediateOperatorState {}))
-    }
-    fn make_runtime_stats_builder(&self) -> Arc<dyn RuntimeStatsBuilder> {
-        Arc::new(BaseStatsBuilder {})
+    fn make_state(&self) -> DaftResult<Self::State>;
+    fn make_runtime_stats(&self) -> Arc<dyn RuntimeStats> {
+        Arc::new(DefaultRuntimeStats::default())
     }
     /// The maximum number of concurrent workers that can be spawned for this operator.
     /// Each worker will has its own IntermediateOperatorState.
@@ -84,43 +77,34 @@ pub trait IntermediateOperator: Send + Sync {
     }
 }
 
-pub struct IntermediateNode {
-    intermediate_op: Arc<dyn IntermediateOperator>,
+pub struct IntermediateNode<Op: IntermediateOperator> {
+    intermediate_op: Arc<Op>,
     children: Vec<Box<dyn PipelineNode>>,
-    runtime_stats: Arc<RuntimeStatsContext>,
+    runtime_stats: Arc<dyn RuntimeStats>,
     plan_stats: StatsState,
-    node_info: NodeInfo,
+    node_info: Arc<NodeInfo>,
 }
 
-impl IntermediateNode {
+impl<Op: IntermediateOperator + 'static> IntermediateNode<Op> {
     pub(crate) fn new(
-        intermediate_op: Arc<dyn IntermediateOperator>,
+        intermediate_op: Arc<Op>,
         children: Vec<Box<dyn PipelineNode>>,
         plan_stats: StatsState,
         ctx: &RuntimeContext,
     ) -> Self {
-        let info = ctx.next_node_info(intermediate_op.name());
-
-        let rts = RuntimeStatsContext::new_with_builder(
-            info.clone(),
-            intermediate_op.make_runtime_stats_builder(),
+        let info = ctx.next_node_info(
+            Arc::from(intermediate_op.name()),
+            intermediate_op.op_type(),
+            NodeCategory::Intermediate,
         );
-        Self::new_with_runtime_stats(intermediate_op, children, rts, plan_stats, info)
-    }
+        let runtime_stats = intermediate_op.make_runtime_stats();
 
-    pub(crate) fn new_with_runtime_stats(
-        intermediate_op: Arc<dyn IntermediateOperator>,
-        children: Vec<Box<dyn PipelineNode>>,
-        runtime_stats: Arc<RuntimeStatsContext>,
-        plan_stats: StatsState,
-        node_info: NodeInfo,
-    ) -> Self {
         Self {
             intermediate_op,
             children,
             runtime_stats,
             plan_stats,
-            node_info,
+            node_info: Arc::new(info),
         }
     }
 
@@ -130,22 +114,16 @@ impl IntermediateNode {
 
     #[instrument(level = "info", skip_all, name = "IntermediateOperator::run_worker")]
     pub async fn run_worker(
-        op: Arc<dyn IntermediateOperator>,
+        op: Arc<Op>,
         receiver: Receiver<Arc<MicroPartition>>,
         sender: Sender<Arc<MicroPartition>>,
-        rt_context: Arc<RuntimeStatsContext>,
-        rt_stats_handler: Arc<RuntimeStatsEventHandler>,
+        runtime_stats: Arc<dyn RuntimeStats>,
         memory_manager: Arc<MemoryManager>,
     ) -> DaftResult<()> {
         let span = info_span!("IntermediateOp::execute");
         let compute_runtime = get_compute_runtime();
-        let task_spawner = ExecutionTaskSpawner::new(
-            compute_runtime,
-            memory_manager,
-            rt_context,
-            rt_stats_handler,
-            span,
-        );
+        let task_spawner =
+            ExecutionTaskSpawner::new(compute_runtime, memory_manager, runtime_stats.clone(), span);
         let mut state = op.make_state()?;
         while let Some(morsel) = receiver.recv().await {
             loop {
@@ -188,17 +166,16 @@ impl IntermediateNode {
                     input_receiver,
                     output_sender,
                     self.runtime_stats.clone(),
-                    runtime_handle.runtime_stats_handler(),
                     memory_manager.clone(),
                 ),
-                self.intermediate_op.name(),
+                &self.intermediate_op.name(),
             );
         }
         output_receiver
     }
 }
 
-impl TreeDisplay for IntermediateNode {
+impl<Op: IntermediateOperator + 'static> TreeDisplay for IntermediateNode<Op> {
     fn display_as(&self, level: common_display::DisplayLevel) -> String {
         use std::fmt::Write;
         let mut display = String::new();
@@ -216,7 +193,7 @@ impl TreeDisplay for IntermediateNode {
                 }
                 if matches!(level, DisplayLevel::Verbose) {
                     writeln!(display).unwrap();
-                    let rt_result = self.runtime_stats.render();
+                    let rt_result = self.runtime_stats.snapshot();
                     for (name, value) in rt_result {
                         writeln!(display, "{} = {}", name.capitalize(), value).unwrap();
                     }
@@ -231,7 +208,7 @@ impl TreeDisplay for IntermediateNode {
     }
 }
 
-impl PipelineNode for IntermediateNode {
+impl<Op: IntermediateOperator + 'static> PipelineNode for IntermediateNode<Op> {
     fn children(&self) -> Vec<&dyn PipelineNode> {
         self.children
             .iter()
@@ -239,8 +216,12 @@ impl PipelineNode for IntermediateNode {
             .collect()
     }
 
-    fn name(&self) -> &'static str {
-        self.intermediate_op.name()
+    fn boxed_children(&self) -> Vec<&Box<dyn PipelineNode>> {
+        self.children.iter().collect()
+    }
+
+    fn name(&self) -> Arc<str> {
+        self.node_info.name.clone()
     }
 
     fn start(
@@ -249,33 +230,22 @@ impl PipelineNode for IntermediateNode {
         runtime_handle: &mut ExecutionRuntimeContext,
     ) -> crate::Result<Receiver<Arc<MicroPartition>>> {
         let mut child_result_receivers = Vec::with_capacity(self.children.len());
-        let progress_bar = runtime_handle.make_progress_bar(
-            self.name(),
-            ProgressBarColor::Magenta,
-            self.node_id(),
-            self.runtime_stats.clone(),
-        );
 
         for child in &self.children {
             let child_result_receiver = child.start(maintain_order, runtime_handle)?;
-            child_result_receivers.push(CountingReceiver::new(
+            child_result_receivers.push(InitializingCountingReceiver::new(
                 child_result_receiver,
+                self.node_id(),
                 self.runtime_stats.clone(),
-                progress_bar.clone(),
-                runtime_handle.runtime_stats_handler(),
+                runtime_handle.stats_manager(),
             ));
         }
         let op = self.intermediate_op.clone();
         let num_workers = op.max_concurrency().context(PipelineExecutionSnafu {
-            node_name: self.name(),
+            node_name: self.name().to_string(),
         })?;
         let (destination_sender, destination_receiver) = create_channel(0);
-        let counting_sender = CountingSender::new(
-            destination_sender,
-            self.runtime_stats.clone(),
-            progress_bar,
-            runtime_handle.runtime_stats_handler(),
-        );
+        let counting_sender = CountingSender::new(destination_sender, self.runtime_stats.clone());
 
         let dispatch_spawner = self
             .intermediate_op
@@ -287,7 +257,7 @@ impl PipelineNode for IntermediateNode {
         );
         runtime_handle.spawn_local(
             async move { spawned_dispatch_result.spawned_dispatch_task.await? },
-            self.name(),
+            &self.name(),
         );
 
         let mut output_receiver = self.spawn_workers(
@@ -296,6 +266,8 @@ impl PipelineNode for IntermediateNode {
             maintain_order,
             runtime_handle.memory_manager(),
         );
+        let stats_manager = runtime_handle.stats_manager();
+        let node_id = self.node_id();
         runtime_handle.spawn_local(
             async move {
                 while let Some(morsel) = output_receiver.recv().await {
@@ -303,9 +275,10 @@ impl PipelineNode for IntermediateNode {
                         return Ok(());
                     }
                 }
+                stats_manager.finalize_node(node_id);
                 Ok(())
             },
-            op.name(),
+            &op.name(),
         );
         Ok(destination_receiver)
     }
@@ -320,4 +293,12 @@ impl PipelineNode for IntermediateNode {
     fn plan_id(&self) -> Arc<str> {
         Arc::from(self.node_info.context.get("plan_id").unwrap().clone())
     }
+
+    fn node_info(&self) -> Arc<NodeInfo> {
+        self.node_info.clone()
+    }
+
+    fn runtime_stats(&self) -> Arc<dyn RuntimeStats> {
+        self.runtime_stats.clone()
+    }
 }
