@@ -10,11 +10,11 @@ use tracing::{instrument, Span};
 
 use super::blocking_sink::{
     BlockingSink, BlockingSinkFinalizeOutput, BlockingSinkFinalizeResult, BlockingSinkSinkResult,
-    BlockingSinkState, BlockingSinkStatus,
+    BlockingSinkStatus,
 };
-use crate::ExecutionTaskSpawner;
+use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
-enum AggregateState {
+pub(crate) enum AggregateState {
     Accumulating(Vec<Arc<MicroPartition>>),
     Done,
 }
@@ -39,12 +39,6 @@ impl AggregateState {
     }
 }
 
-impl BlockingSinkState for AggregateState {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any {
-        self
-    }
-}
-
 struct AggParams {
     sink_agg_exprs: Vec<BoundAggExpr>,
     finalize_agg_exprs: Vec<BoundAggExpr>,
@@ -52,15 +46,23 @@ struct AggParams {
 }
 
 pub struct AggregateSink {
+    aggregate_name: &'static str,
     agg_sink_params: Arc<AggParams>,
 }
 
 impl AggregateSink {
     pub fn new(aggregations: &[BoundAggExpr], input_schema: &SchemaRef) -> DaftResult<Self> {
+        let aggregate_name = if aggregations.len() == 1 {
+            aggregations[0].as_ref().agg_name()
+        } else {
+            "Aggregate"
+        };
+
         let (sink_agg_exprs, finalize_agg_exprs, final_projections) =
             daft_physical_plan::populate_aggregation_stages_bound(aggregations, input_schema, &[])?;
 
         Ok(Self {
+            aggregate_name,
             agg_sink_params: Arc::new(AggParams {
                 sink_agg_exprs,
                 finalize_agg_exprs,
@@ -71,23 +73,21 @@ impl AggregateSink {
 }
 
 impl BlockingSink for AggregateSink {
+    type State = AggregateState;
+
     #[instrument(skip_all, name = "AggregateSink::sink")]
     fn sink(
         &self,
         input: Arc<MicroPartition>,
-        mut state: Box<dyn BlockingSinkState>,
+        mut state: Self::State,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkSinkResult {
+    ) -> BlockingSinkSinkResult<Self> {
         let params = self.agg_sink_params.clone();
         spawner
             .spawn(
                 async move {
-                    let agg_state = state
-                        .as_any_mut()
-                        .downcast_mut::<AggregateState>()
-                        .expect("AggregateSink should have AggregateState");
                     let agged = Arc::new(input.agg(&params.sink_agg_exprs, &[])?);
-                    agg_state.push(agged);
+                    state.push(agged);
                     Ok(BlockingSinkStatus::NeedMoreInput(state))
                 },
                 Span::current(),
@@ -98,20 +98,14 @@ impl BlockingSink for AggregateSink {
     #[instrument(skip_all, name = "AggregateSink::finalize")]
     fn finalize(
         &self,
-        states: Vec<Box<dyn BlockingSinkState>>,
+        states: Vec<Self::State>,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkFinalizeResult {
+    ) -> BlockingSinkFinalizeResult<Self> {
         let params = self.agg_sink_params.clone();
         spawner
             .spawn(
                 async move {
-                    let all_parts = states.into_iter().flat_map(|mut state| {
-                        state
-                            .as_any_mut()
-                            .downcast_mut::<AggregateState>()
-                            .expect("AggregateSink should have AggregateState")
-                            .finalize()
-                    });
+                    let all_parts = states.into_iter().flat_map(|mut state| state.finalize());
                     let concated = MicroPartition::concat(all_parts)?;
                     let agged = concated.agg(&params.finalize_agg_exprs, &[])?;
                     let projected = agged.eval_expression_list(&params.final_projections)?;
@@ -124,8 +118,12 @@ impl BlockingSink for AggregateSink {
             .into()
     }
 
-    fn name(&self) -> &'static str {
-        "Aggregate"
+    fn name(&self) -> NodeName {
+        self.aggregate_name.into()
+    }
+
+    fn op_type(&self) -> NodeType {
+        NodeType::Aggregate
     }
 
     fn multiline_display(&self) -> Vec<String> {
@@ -143,7 +141,7 @@ impl BlockingSink for AggregateSink {
         get_compute_pool_num_threads()
     }
 
-    fn make_state(&self) -> DaftResult<Box<dyn BlockingSinkState>> {
-        Ok(Box::new(AggregateState::Accumulating(vec![])))
+    fn make_state(&self) -> DaftResult<Self::State> {
+        Ok(AggregateState::Accumulating(vec![]))
     }
 }
