@@ -11,53 +11,57 @@ use tracing::{info_span, instrument};
 use crate::{
     channel::{create_channel, Receiver},
     dispatcher::{DispatchSpawner, UnorderedDispatcher},
-    pipeline::{NodeInfo, PipelineNode, RuntimeContext},
-    progress_bar::ProgressBarColor,
+    ops::{NodeCategory, NodeInfo, NodeType},
+    pipeline::{NodeName, PipelineNode, RuntimeContext},
     resource_manager::MemoryManager,
     runtime_stats::{
-        BaseStatsBuilder, CountingReceiver, CountingSender, RuntimeStatsBuilder,
-        RuntimeStatsContext, RuntimeStatsEventHandler,
+        CountingSender, DefaultRuntimeStats, InitializingCountingReceiver, RuntimeStats,
     },
     ExecutionRuntimeContext, ExecutionTaskSpawner, OperatorOutput, TaskSet,
 };
-pub trait BlockingSinkState: Send + Sync {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any;
-}
 
-pub enum BlockingSinkStatus {
-    NeedMoreInput(Box<dyn BlockingSinkState>),
+pub enum BlockingSinkStatus<Op: BlockingSink> {
+    NeedMoreInput(Op::State),
     #[allow(dead_code)]
-    Finished(Box<dyn BlockingSinkState>),
+    Finished(Op::State),
 }
 
-pub enum BlockingSinkFinalizeOutput {
+pub enum BlockingSinkFinalizeOutput<Op: BlockingSink> {
     #[allow(dead_code)]
     HasMoreOutput {
-        states: Vec<Box<dyn BlockingSinkState>>,
+        states: Vec<Op::State>,
         output: Vec<Arc<MicroPartition>>,
     },
     Finished(Vec<Arc<MicroPartition>>),
 }
 
-pub(crate) type BlockingSinkSinkResult = OperatorOutput<DaftResult<BlockingSinkStatus>>;
-pub(crate) type BlockingSinkFinalizeResult = OperatorOutput<DaftResult<BlockingSinkFinalizeOutput>>;
-pub trait BlockingSink: Send + Sync {
+pub(crate) type BlockingSinkSinkResult<Op> = OperatorOutput<DaftResult<BlockingSinkStatus<Op>>>;
+pub(crate) type BlockingSinkFinalizeResult<Op> =
+    OperatorOutput<DaftResult<BlockingSinkFinalizeOutput<Op>>>;
+pub(crate) trait BlockingSink: Send + Sync {
+    type State: Send + Sync + Unpin;
+
     fn sink(
         &self,
         input: Arc<MicroPartition>,
-        state: Box<dyn BlockingSinkState>,
+        state: Self::State,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkSinkResult;
+    ) -> BlockingSinkSinkResult<Self>
+    where
+        Self: Sized;
     fn finalize(
         &self,
-        states: Vec<Box<dyn BlockingSinkState>>,
+        states: Vec<Self::State>,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkFinalizeResult;
-    fn name(&self) -> &'static str;
+    ) -> BlockingSinkFinalizeResult<Self>
+    where
+        Self: Sized;
+    fn name(&self) -> NodeName;
+    fn op_type(&self) -> NodeType;
     fn multiline_display(&self) -> Vec<String>;
-    fn make_state(&self) -> DaftResult<Box<dyn BlockingSinkState>>;
-    fn make_runtime_stats_builder(&self) -> Arc<dyn RuntimeStatsBuilder> {
-        Arc::new(BaseStatsBuilder {})
+    fn make_state(&self) -> DaftResult<Self::State>;
+    fn make_runtime_stats(&self) -> Arc<dyn RuntimeStats> {
+        Arc::new(DefaultRuntimeStats::default())
     }
     fn dispatch_spawner(
         &self,
@@ -72,36 +76,31 @@ pub trait BlockingSink: Send + Sync {
     }
 }
 
-pub struct BlockingSinkNode {
-    op: Arc<dyn BlockingSink>,
-    name: &'static str,
+pub struct BlockingSinkNode<Op: BlockingSink> {
+    op: Arc<Op>,
     child: Box<dyn PipelineNode>,
-    runtime_stats: Arc<RuntimeStatsContext>,
+    runtime_stats: Arc<dyn RuntimeStats>,
     plan_stats: StatsState,
-    node_info: NodeInfo,
+    node_info: Arc<NodeInfo>,
 }
 
-impl BlockingSinkNode {
+impl<Op: BlockingSink + 'static> BlockingSinkNode<Op> {
     pub(crate) fn new(
-        op: Arc<dyn BlockingSink>,
+        op: Arc<Op>,
         child: Box<dyn PipelineNode>,
         plan_stats: StatsState,
         ctx: &RuntimeContext,
     ) -> Self {
-        let name = op.name();
-        let node_info = ctx.next_node_info(name);
+        let name = op.name().into();
+        let node_info = ctx.next_node_info(name, op.op_type(), NodeCategory::BlockingSink);
+        let runtime_stats = op.make_runtime_stats();
 
-        let runtime_stats = RuntimeStatsContext::new_with_builder(
-            node_info.clone(),
-            op.make_runtime_stats_builder(),
-        );
         Self {
             op,
-            name,
             child,
             runtime_stats,
             plan_stats,
-            node_info,
+            node_info: Arc::new(node_info),
         }
     }
     pub(crate) fn boxed(self) -> Box<dyn PipelineNode> {
@@ -110,21 +109,15 @@ impl BlockingSinkNode {
 
     #[instrument(level = "info", skip_all, name = "BlockingSink::run_worker")]
     async fn run_worker(
-        op: Arc<dyn BlockingSink>,
+        op: Arc<Op>,
         input_receiver: Receiver<Arc<MicroPartition>>,
-        rt_context: Arc<RuntimeStatsContext>,
-        rt_stats_handler: Arc<RuntimeStatsEventHandler>,
+        runtime_stats: Arc<dyn RuntimeStats>,
         memory_manager: Arc<MemoryManager>,
-    ) -> DaftResult<Box<dyn BlockingSinkState>> {
+    ) -> DaftResult<Op::State> {
         let span = info_span!("BlockingSink::Sink");
         let compute_runtime = get_compute_runtime();
-        let spawner = ExecutionTaskSpawner::new(
-            compute_runtime,
-            memory_manager,
-            rt_context,
-            rt_stats_handler,
-            span,
-        );
+        let spawner =
+            ExecutionTaskSpawner::new(compute_runtime, memory_manager, runtime_stats, span);
         let mut state = op.make_state()?;
         while let Some(morsel) = input_receiver.recv().await {
             let result = op.sink(morsel, state, &spawner).await??;
@@ -142,26 +135,24 @@ impl BlockingSinkNode {
     }
 
     fn spawn_workers(
-        op: Arc<dyn BlockingSink>,
+        op: Arc<Op>,
         input_receivers: Vec<Receiver<Arc<MicroPartition>>>,
-        task_set: &mut TaskSet<DaftResult<Box<dyn BlockingSinkState>>>,
-        stats: Arc<RuntimeStatsContext>,
-        stats_handler: Arc<RuntimeStatsEventHandler>,
+        task_set: &mut TaskSet<DaftResult<Op::State>>,
+        runtime_stats: Arc<dyn RuntimeStats>,
         memory_manager: Arc<MemoryManager>,
     ) {
         for input_receiver in input_receivers {
             task_set.spawn(Self::run_worker(
                 op.clone(),
                 input_receiver,
-                stats.clone(),
-                stats_handler.clone(),
+                runtime_stats.clone(),
                 memory_manager.clone(),
             ));
         }
     }
 }
 
-impl TreeDisplay for BlockingSinkNode {
+impl<Op: BlockingSink> TreeDisplay for BlockingSinkNode<Op> {
     fn display_as(&self, level: common_display::DisplayLevel) -> String {
         use std::fmt::Write;
         let mut display = String::new();
@@ -178,7 +169,7 @@ impl TreeDisplay for BlockingSinkNode {
                     writeln!(display, "Stats = {}", stats).unwrap();
                 }
                 if matches!(level, DisplayLevel::Verbose) {
-                    let rt_result = self.runtime_stats.render();
+                    let rt_result = self.runtime_stats.snapshot();
                     for (name, value) in rt_result {
                         writeln!(display, "{} = {}", name.capitalize(), value).unwrap();
                     }
@@ -193,13 +184,17 @@ impl TreeDisplay for BlockingSinkNode {
     }
 }
 
-impl PipelineNode for BlockingSinkNode {
+impl<Op: BlockingSink + 'static> PipelineNode for BlockingSinkNode<Op> {
     fn children(&self) -> Vec<&dyn PipelineNode> {
         vec![self.child.as_ref()]
     }
 
-    fn name(&self) -> &'static str {
-        self.name
+    fn boxed_children(&self) -> Vec<&Box<dyn PipelineNode>> {
+        vec![&self.child]
+    }
+
+    fn name(&self) -> Arc<str> {
+        self.node_info.name.clone()
     }
 
     fn start(
@@ -207,27 +202,16 @@ impl PipelineNode for BlockingSinkNode {
         _maintain_order: bool,
         runtime_handle: &mut ExecutionRuntimeContext,
     ) -> crate::Result<Receiver<Arc<MicroPartition>>> {
-        let progress_bar = runtime_handle.make_progress_bar(
-            self.name(),
-            ProgressBarColor::Cyan,
+        let child_results_receiver = self.child.start(false, runtime_handle)?;
+        let counting_receiver = InitializingCountingReceiver::new(
+            child_results_receiver,
             self.node_id(),
             self.runtime_stats.clone(),
-        );
-        let child_results_receiver = self.child.start(false, runtime_handle)?;
-        let counting_receiver = CountingReceiver::new(
-            child_results_receiver,
-            self.runtime_stats.clone(),
-            progress_bar.clone(),
-            runtime_handle.runtime_stats_handler(),
+            runtime_handle.stats_manager(),
         );
 
         let (destination_sender, destination_receiver) = create_channel(0);
-        let counting_sender = CountingSender::new(
-            destination_sender,
-            self.runtime_stats.clone(),
-            progress_bar,
-            runtime_handle.runtime_stats_handler(),
-        );
+        let counting_sender = CountingSender::new(destination_sender, self.runtime_stats.clone());
 
         let op = self.op.clone();
         let runtime_stats = self.runtime_stats.clone();
@@ -241,11 +225,12 @@ impl PipelineNode for BlockingSinkNode {
         );
         runtime_handle.spawn_local(
             async move { spawned_dispatch_result.spawned_dispatch_task.await? },
-            self.name(),
+            &self.name(),
         );
 
         let memory_manager = runtime_handle.memory_manager();
-        let rt_stats_handler = runtime_handle.runtime_stats_handler();
+        let stats_manager = runtime_handle.stats_manager();
+        let node_id = self.node_id();
         runtime_handle.spawn_local(
             async move {
                 let mut task_set = TaskSet::new();
@@ -254,7 +239,6 @@ impl PipelineNode for BlockingSinkNode {
                     spawned_dispatch_result.worker_receivers,
                     &mut task_set,
                     runtime_stats.clone(),
-                    rt_stats_handler.clone(),
                     memory_manager.clone(),
                 );
 
@@ -269,7 +253,6 @@ impl PipelineNode for BlockingSinkNode {
                     compute_runtime,
                     memory_manager,
                     runtime_stats.clone(),
-                    rt_stats_handler,
                     info_span!("BlockingSink::Finalize"),
                 );
                 loop {
@@ -289,9 +272,11 @@ impl PipelineNode for BlockingSinkNode {
                         }
                     }
                 }
+
+                stats_manager.finalize_node(node_id);
                 Ok(())
             },
-            self.name(),
+            &self.name(),
         );
         Ok(destination_receiver)
     }
@@ -304,4 +289,10 @@ impl PipelineNode for BlockingSinkNode {
     fn plan_id(&self) -> Arc<str> {
         Arc::from(self.node_info.context.get("plan_id").unwrap().clone())
     }
+    fn node_info(&self) -> Arc<NodeInfo> {
+        self.node_info.clone()
+    }
+    fn runtime_stats(&self) -> Arc<dyn RuntimeStats> {
+        self.runtime_stats.clone()
+    }
 }
