@@ -12,11 +12,11 @@ use tracing::{instrument, Span};
 
 use super::blocking_sink::{
     BlockingSink, BlockingSinkFinalizeOutput, BlockingSinkFinalizeResult, BlockingSinkSinkResult,
-    BlockingSinkState, BlockingSinkStatus,
+    BlockingSinkStatus,
 };
-use crate::ExecutionTaskSpawner;
+use crate::{ops::NodeType, pipeline::NodeName, ExecutionTaskSpawner};
 
-struct CommitWriteState {
+pub(crate) struct CommitWriteState {
     written_file_path_record_batches: Vec<RecordBatch>,
 }
 
@@ -32,12 +32,6 @@ impl CommitWriteState {
     }
 }
 
-impl BlockingSinkState for CommitWriteState {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any {
-        self
-    }
-}
-
 pub(crate) struct CommitWriteSink {
     file_schema: SchemaRef,
     file_info: OutputFileInfo<BoundExpr>,
@@ -53,33 +47,31 @@ impl CommitWriteSink {
 }
 
 impl BlockingSink for CommitWriteSink {
+    type State = CommitWriteState;
+
     #[instrument(skip_all, name = "CommitWriteSink::sink")]
     fn sink(
         &self,
         input: Arc<MicroPartition>,
-        mut state: Box<dyn BlockingSinkState>,
+        mut state: Self::State,
         _spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkSinkResult {
+    ) -> BlockingSinkSinkResult<Self> {
         let tables = match input.get_tables() {
             Ok(tables) => tables,
             Err(e) => {
                 return Err(e.into()).into();
             }
         };
-        state
-            .as_any_mut()
-            .downcast_mut::<CommitWriteState>()
-            .expect("CommitWriteSink should have CommitWriteState")
-            .append(tables.iter().cloned());
+        state.append(tables.iter().cloned());
         Ok(BlockingSinkStatus::NeedMoreInput(state)).into()
     }
 
     #[instrument(skip_all, name = "WriteSink::finalize")]
     fn finalize(
         &self,
-        states: Vec<Box<dyn BlockingSinkState>>,
+        states: Vec<Self::State>,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkFinalizeResult {
+    ) -> BlockingSinkFinalizeResult<Self> {
         let file_schema = self.file_schema.clone();
         let file_info = self.file_info.clone();
         spawner
@@ -88,10 +80,6 @@ impl BlockingSink for CommitWriteSink {
                     let written_file_path_record_batches = states
                         .into_iter()
                         .flat_map(|mut state| {
-                            let state = state
-                                .as_any_mut()
-                                .downcast_mut::<CommitWriteState>()
-                                .expect("State type mismatch");
                             std::mem::take(&mut state.written_file_path_record_batches)
                         })
                         .collect::<Vec<_>>();
@@ -162,12 +150,16 @@ impl BlockingSink for CommitWriteSink {
             .into()
     }
 
-    fn name(&self) -> &'static str {
-        "CommitWriteSink"
+    fn name(&self) -> NodeName {
+        "CommitWriteSink".into()
     }
 
-    fn make_state(&self) -> DaftResult<Box<dyn BlockingSinkState>> {
-        Ok(Box::new(CommitWriteState::new()) as Box<dyn BlockingSinkState>)
+    fn op_type(&self) -> NodeType {
+        NodeType::CommitWrite
+    }
+
+    fn make_state(&self) -> DaftResult<Self::State> {
+        Ok(CommitWriteState::new())
     }
 
     fn multiline_display(&self) -> Vec<String> {
