@@ -1,51 +1,65 @@
-use std::sync::{atomic::AtomicU64, Arc};
+use std::{
+    sync::{
+        atomic::{AtomicU64, Ordering},
+        Arc,
+    },
+    time::Duration,
+};
 
 use common_error::DaftResult;
+use common_metrics::{snapshot, Stat, StatSnapshotSend};
 use common_runtime::get_compute_pool_num_threads;
 use daft_core::prelude::SchemaRef;
 use daft_dsl::expr::bound_expr::BoundExpr;
 use daft_micropartition::MicroPartition;
 use daft_recordbatch::RecordBatch;
 use daft_writers::{AsyncFileWriter, WriterFactory};
-use indexmap::IndexMap;
-use indicatif::{HumanBytes, HumanCount};
 use tracing::{instrument, Span};
 
 use super::blocking_sink::{
     BlockingSink, BlockingSinkFinalizeOutput, BlockingSinkFinalizeResult, BlockingSinkSinkResult,
-    BlockingSinkState, BlockingSinkStatus,
+    BlockingSinkStatus,
 };
 use crate::{
     dispatcher::{DispatchSpawner, PartitionedDispatcher, UnorderedDispatcher},
-    runtime_stats::{RuntimeStatsBuilder, ROWS_EMITTED_KEY, ROWS_RECEIVED_KEY},
+    ops::NodeType,
+    pipeline::NodeName,
+    runtime_stats::{RuntimeStats, CPU_US_KEY, ROWS_EMITTED_KEY, ROWS_RECEIVED_KEY},
     ExecutionRuntimeContext, ExecutionTaskSpawner,
 };
 
-struct WriteStatsBuilder {
+#[derive(Default)]
+struct WriteStats {
+    cpu_us: AtomicU64,
+    rows_received: AtomicU64,
+    rows_emitted: AtomicU64, // TODO: Remove or rename to files written?
     bytes_written: AtomicU64,
 }
 
-impl RuntimeStatsBuilder for WriteStatsBuilder {
+impl RuntimeStats for WriteStats {
     fn as_any_arc(self: Arc<Self>) -> Arc<dyn std::any::Any + Send + Sync> {
         self
     }
 
-    fn build(
-        &self,
-        stats: &mut IndexMap<&'static str, String>,
-        rows_received: u64,
-        rows_emitted: u64,
-    ) {
-        stats.insert(ROWS_RECEIVED_KEY, HumanCount(rows_received).to_string());
-        stats.insert(ROWS_EMITTED_KEY, HumanCount(rows_emitted).to_string());
-        stats.insert(
-            "bytes written",
-            HumanBytes(
-                self.bytes_written
-                    .load(std::sync::atomic::Ordering::Relaxed),
-            )
-            .to_string(),
-        );
+    fn build_snapshot(&self, ordering: Ordering) -> StatSnapshotSend {
+        snapshot![
+            CPU_US_KEY; Stat::Duration(Duration::from_micros(self.cpu_us.load(ordering))),
+            ROWS_RECEIVED_KEY; Stat::Count(self.rows_received.load(ordering)),
+            ROWS_EMITTED_KEY; Stat::Count(self.rows_emitted.load(ordering)),
+            "bytes written"; Stat::Bytes(self.bytes_written.load(ordering)),
+        ]
+    }
+
+    fn add_rows_received(&self, rows: u64) {
+        self.rows_received.fetch_add(rows, Ordering::Relaxed);
+    }
+
+    fn add_rows_emitted(&self, rows: u64) {
+        self.rows_emitted.fetch_add(rows, Ordering::Relaxed);
+    }
+
+    fn add_cpu_us(&self, cpu_us: u64) {
+        self.cpu_us.fetch_add(cpu_us, Ordering::Relaxed);
     }
 }
 
@@ -62,10 +76,10 @@ pub enum WriteFormat {
     Deltalake,
     PartitionedDeltalake,
     Lance,
-    DataSink,
+    DataSink(String),
 }
 
-struct WriteState {
+pub(crate) struct WriteState {
     writer: Box<dyn AsyncFileWriter<Input = Arc<MicroPartition>, Result = Vec<RecordBatch>>>,
 }
 
@@ -77,12 +91,6 @@ impl WriteState {
     }
 }
 
-impl BlockingSinkState for WriteState {
-    fn as_any_mut(&mut self) -> &mut dyn std::any::Any {
-        self
-    }
-}
-
 pub(crate) struct WriteSink {
     write_format: WriteFormat,
     writer_factory: Arc<dyn WriterFactory<Input = Arc<MicroPartition>, Result = Vec<RecordBatch>>>,
@@ -109,30 +117,26 @@ impl WriteSink {
 }
 
 impl BlockingSink for WriteSink {
+    type State = WriteState;
+
     #[instrument(skip_all, name = "WriteSink::sink")]
     fn sink(
         &self,
         input: Arc<MicroPartition>,
-        mut state: Box<dyn BlockingSinkState>,
+        mut state: Self::State,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkSinkResult {
-        let builder = spawner.runtime_context.builder.clone();
+    ) -> BlockingSinkSinkResult<Self> {
+        let builder = spawner.runtime_stats.clone();
 
         spawner
             .spawn(
                 async move {
-                    let bytes_written = state
-                        .as_any_mut()
-                        .downcast_mut::<WriteState>()
-                        .expect("WriteSink should have WriteState")
-                        .writer
-                        .write(input)
-                        .await?;
+                    let bytes_written = state.writer.write(input).await?;
 
                     builder
                         .as_any_arc()
-                        .downcast_ref::<WriteStatsBuilder>()
-                        .expect("WriteStatsBuilder should be the additional stats builder")
+                        .downcast_ref::<WriteStats>()
+                        .expect("WriteStats should be the additional stats builder")
                         .bytes_written
                         .fetch_add(bytes_written as u64, std::sync::atomic::Ordering::Relaxed);
 
@@ -146,19 +150,15 @@ impl BlockingSink for WriteSink {
     #[instrument(skip_all, name = "WriteSink::finalize")]
     fn finalize(
         &self,
-        states: Vec<Box<dyn BlockingSinkState>>,
+        states: Vec<Self::State>,
         spawner: &ExecutionTaskSpawner,
-    ) -> BlockingSinkFinalizeResult {
+    ) -> BlockingSinkFinalizeResult<Self> {
         let file_schema = self.file_schema.clone();
         spawner
             .spawn(
                 async move {
                     let mut results = vec![];
                     for mut state in states {
-                        let state = state
-                            .as_any_mut()
-                            .downcast_mut::<WriteState>()
-                            .expect("State type mismatch");
                         results.extend(state.writer.close().await?);
                     }
                     let mp = Arc::new(MicroPartition::new_loaded(
@@ -173,32 +173,34 @@ impl BlockingSink for WriteSink {
             .into()
     }
 
-    fn name(&self) -> &'static str {
-        match self.write_format {
-            WriteFormat::Parquet => "ParquetSink",
-            WriteFormat::PartitionedParquet => "PartitionedParquetSink",
-            WriteFormat::Csv => "CsvSink",
-            WriteFormat::PartitionedCsv => "PartitionedCsvSink",
-            WriteFormat::Json => "JsonSink",
-            WriteFormat::PartitionedJson => "PartitionedJsonSink",
-            WriteFormat::Iceberg => "IcebergSink",
-            WriteFormat::PartitionedIceberg => "PartitionedIcebergSink",
-            WriteFormat::Deltalake => "DeltalakeSink",
-            WriteFormat::PartitionedDeltalake => "PartitionedDeltalakeSink",
-            WriteFormat::Lance => "LanceSink",
-            WriteFormat::DataSink => "DataSink",
+    fn name(&self) -> NodeName {
+        match &self.write_format {
+            WriteFormat::Parquet => "Parquet Write".into(),
+            WriteFormat::PartitionedParquet => "PartitionedParquet Write".into(),
+            WriteFormat::Csv => "Csv Write".into(),
+            WriteFormat::PartitionedCsv => "PartitionedCsv Write".into(),
+            WriteFormat::Json => "Json Write".into(),
+            WriteFormat::PartitionedJson => "PartitionedJson Write".into(),
+            WriteFormat::Iceberg => "Iceberg Write".into(),
+            WriteFormat::PartitionedIceberg => "PartitionedIceberg Write".into(),
+            WriteFormat::Deltalake => "Deltalake Write".into(),
+            WriteFormat::PartitionedDeltalake => "PartitionedDeltalake Write".into(),
+            WriteFormat::Lance => "Lance Write".into(),
+            WriteFormat::DataSink(name) => name.clone().into(),
         }
     }
 
-    fn make_state(&self) -> DaftResult<Box<dyn BlockingSinkState>> {
-        let writer = self.writer_factory.create_writer(0, None)?;
-        Ok(Box::new(WriteState::new(writer)) as Box<dyn BlockingSinkState>)
+    fn op_type(&self) -> NodeType {
+        NodeType::Write
     }
 
-    fn make_runtime_stats_builder(&self) -> Arc<dyn RuntimeStatsBuilder> {
-        Arc::new(WriteStatsBuilder {
-            bytes_written: AtomicU64::new(0),
-        })
+    fn make_state(&self) -> DaftResult<Self::State> {
+        let writer = self.writer_factory.create_writer(0, None)?;
+        Ok(WriteState::new(writer))
+    }
+
+    fn make_runtime_stats(&self) -> Arc<dyn RuntimeStats> {
+        Arc::new(WriteStats::default())
     }
 
     fn dispatch_spawner(
