@@ -12,11 +12,11 @@ pub use plan::DistributedActorPoolProject;
 #[cfg(feature = "python")]
 pub use plan::LanceWrite;
 pub use plan::{
-    ActorPoolProject, CommitWrite, Concat, CrossJoin, Dedup, EmptyScan, Explode, Filter,
-    HashAggregate, HashJoin, InMemoryScan, Limit, LocalPhysicalPlan, LocalPhysicalPlanRef,
+    CommitWrite, Concat, CrossJoin, Dedup, EmptyScan, Explode, Filter, HashAggregate, HashJoin,
+    InMemoryScan, IntoBatches, Limit, LocalPhysicalPlan, LocalPhysicalPlanRef,
     MonotonicallyIncreasingId, PhysicalScan, PhysicalWrite, Pivot, Project, Repartition, Sample,
-    Sort, TopN, UnGroupedAggregate, Unpivot, WindowOrderByOnly, WindowPartitionAndDynamicFrame,
-    WindowPartitionAndOrderBy, WindowPartitionOnly,
+    Sort, TopN, UDFProject, UnGroupedAggregate, Unpivot, WindowOrderByOnly,
+    WindowPartitionAndDynamicFrame, WindowPartitionAndOrderBy, WindowPartitionOnly,
 };
 #[cfg(feature = "python")]
 pub use python::{register_modules, PyLocalPhysicalPlan};
