@@ -104,6 +104,7 @@ const V_MIN_COUNT: &str = "__min_count";
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct Intersect {
     pub plan_id: Option<usize>,
+    pub node_id: Option<usize>,
     // Upstream nodes.
     pub lhs: Arc<LogicalPlan>,
     pub rhs: Arc<LogicalPlan>,
@@ -121,6 +122,7 @@ impl Intersect {
         check_structurally_equal(lhs_schema, rhs_schema, "intersect")?;
         Ok(Self {
             plan_id: None,
+            node_id: None,
             lhs,
             rhs,
             is_all,
@@ -132,6 +134,11 @@ impl Intersect {
         self
     }
 
+    pub fn with_node_id(mut self, node_id: usize) -> Self {
+        self.node_id = Some(node_id);
+        self
+    }
+
     /// intersect operations could be represented by other logical plans
     /// for intersect distinct, it could be represented as a semi join + distinct
     /// ```sql
@@ -226,7 +233,7 @@ impl Intersect {
             let fill_and_explodes = left_cols
                 .iter()
                 .map(|column| {
-                    explode(list_fill(resolved_col(V_MIN_COUNT), column.clone()))
+                    explode(list_fill(column.clone(), resolved_col(V_MIN_COUNT)))
                         .alias(column.name())
                 })
                 .collect::<Vec<_>>();
@@ -251,6 +258,7 @@ impl Intersect {
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct Union {
     pub plan_id: Option<usize>,
+    pub node_id: Option<usize>,
     // Upstream nodes.
     pub lhs: Arc<LogicalPlan>,
     pub rhs: Arc<LogicalPlan>,
@@ -287,6 +295,7 @@ impl Union {
         }
         Ok(Self {
             plan_id: None,
+            node_id: None,
             lhs,
             rhs,
             quantifier,
@@ -299,6 +308,11 @@ impl Union {
         self
     }
 
+    pub fn with_node_id(mut self, node_id: usize) -> Self {
+        self.node_id = Some(node_id);
+        self
+    }
+
     /// union could be represented as a concat + distinct
     /// while union all could be represented as a concat
     pub(crate) fn to_logical_plan(&self) -> logical_plan::Result<LogicalPlan> {
@@ -396,6 +410,7 @@ impl Union {
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Except {
     pub plan_id: Option<usize>,
+    pub node_id: Option<usize>,
     // Upstream nodes.
     pub lhs: Arc<LogicalPlan>,
     pub rhs: Arc<LogicalPlan>,
@@ -412,6 +427,7 @@ impl Except {
         check_structurally_equal(lhs_schema, rhs_schema, "except")?;
         Ok(Self {
             plan_id: None,
+            node_id: None,
             lhs,
             rhs,
             is_all,
@@ -423,6 +439,11 @@ impl Except {
         self
     }
 
+    pub fn with_node_id(mut self, node_id: usize) -> Self {
+        self.node_id = Some(node_id);
+        self
+    }
+
     /// except could be represented by other logical plans
     /// for except distinct, it could be represented as a anti join
     /// ```sql
@@ -493,7 +514,7 @@ impl Except {
             let fill_and_explodes = left_cols
                 .iter()
                 .map(|column| {
-                    explode(list_fill(resolved_col(virtual_sum), column.clone()))
+                    explode(list_fill(column.clone(), resolved_col(virtual_sum)))
                         .alias(column.name())
                 })
                 .collect::<Vec<_>>();
