@@ -1,6 +1,8 @@
 mod detect_monotonic_id;
+mod drop_into_batches;
 mod drop_repartition;
 mod eliminate_cross_join;
+mod eliminate_offsets;
 mod eliminate_subquery_alias;
 mod enrich_with_stats;
 mod extract_window_function;
@@ -16,17 +18,20 @@ mod push_down_projection;
 mod push_down_shard;
 mod reorder_joins;
 mod rewrite_count_distinct;
+mod rewrite_offset;
 mod rule;
 mod shard_scans;
 mod simplify_expressions;
 mod simplify_null_filtered_join;
-mod split_actor_pool_projects;
 mod split_explode_from_project;
+mod split_udfs;
 mod unnest_subquery;
 
 pub use detect_monotonic_id::DetectMonotonicId;
+pub use drop_into_batches::DropIntoBatches;
 pub use drop_repartition::DropRepartition;
 pub use eliminate_cross_join::EliminateCrossJoin;
+pub use eliminate_offsets::EliminateOffsets;
 pub use eliminate_subquery_alias::EliminateSubqueryAliasRule;
 pub use enrich_with_stats::EnrichWithStats;
 pub use extract_window_function::ExtractWindowFunction;
@@ -42,10 +47,11 @@ pub use push_down_projection::PushDownProjection;
 pub use push_down_shard::PushDownShard;
 pub use reorder_joins::ReorderJoins;
 pub use rewrite_count_distinct::RewriteCountDistinct;
+pub use rewrite_offset::RewriteOffset;
 pub use rule::OptimizerRule;
 pub use shard_scans::ShardScans;
 pub use simplify_expressions::SimplifyExpressionsRule;
 pub use simplify_null_filtered_join::SimplifyNullFilteredJoin;
-pub use split_actor_pool_projects::SplitActorPoolProjects;
 pub use split_explode_from_project::SplitExplodeFromProject;
+pub use split_udfs::SplitUDFs;
 pub use unnest_subquery::{UnnestPredicateSubquery, UnnestScalarSubquery};
