@@ -24,9 +24,8 @@ use daft_dsl::{
         bound_expr::{BoundAggExpr, BoundExpr},
         BoundColumn,
     },
-    functions::{FunctionArgs, FunctionEvaluator},
-    null_lit, resolved_col, AggExpr, ApproxPercentileParams, Column, Expr, ExprRef, LiteralValue,
-    SketchType,
+    functions::{scalar::ScalarFn, FunctionArgs, FunctionEvaluator},
+    null_lit, resolved_col, AggExpr, ApproxPercentileParams, Column, Expr, ExprRef, SketchType,
 };
 use daft_functions_list::SeriesListExtension;
 use file_info::FileInfos;
@@ -50,7 +49,7 @@ pub mod python;
 #[cfg(feature = "python")]
 pub use python::register_modules;
 use rand::seq::index::sample;
-use repr_html::html_value;
+pub use repr_html::html_value;
 
 #[macro_export]
 macro_rules! value_err {
@@ -182,14 +181,14 @@ impl RecordBatch {
         }
     }
 
-    pub fn empty(schema: Option<SchemaRef>) -> DaftResult<Self> {
+    pub fn empty(schema: Option<SchemaRef>) -> Self {
         let schema = schema.unwrap_or_else(|| Schema::empty().into());
         let mut columns: Vec<Series> = Vec::with_capacity(schema.len());
         for field in schema.as_ref() {
             let series = Series::empty(&field.name, &field.dtype);
             columns.push(series);
         }
-        Ok(Self::new_unchecked(schema, columns, 0))
+        Self::new_unchecked(schema, columns, 0)
     }
 
     /// Create a RecordBatch from a set of columns.
@@ -374,10 +373,8 @@ impl RecordBatch {
         self.take(&indices.into_series())
     }
 
-    pub fn size_bytes(&self) -> DaftResult<usize> {
-        let column_sizes: DaftResult<Vec<usize>> =
-            self.columns.iter().map(|s| s.size_bytes()).collect();
-        Ok(column_sizes?.iter().sum())
+    pub fn size_bytes(&self) -> usize {
+        self.columns.iter().map(|s| s.size_bytes()).sum()
     }
 
     pub fn filter(&self, predicate: &[BoundExpr]) -> DaftResult<Self> {
@@ -443,7 +440,7 @@ impl RecordBatch {
         schema: Option<SchemaRef>,
     ) -> DaftResult<Self> {
         if tables.is_empty() {
-            return Self::empty(schema);
+            return Ok(Self::empty(schema));
         }
         Self::concat(tables)
     }
@@ -518,6 +515,24 @@ impl RecordBatch {
         &self.columns
     }
 
+    pub fn append_column(&self, series: Series) -> DaftResult<Self> {
+        if self.num_rows != series.len() {
+            return Err(DaftError::ValueError(format!(
+                "Cannot append column to RecordBatch of length {} with column of length {}",
+                self.num_rows,
+                series.len()
+            )));
+        }
+
+        let mut new_schema = self.schema.as_ref().clone();
+        new_schema.append(Field::new(series.name(), series.data_type().clone()));
+
+        let mut new_columns = self.columns.as_ref().clone();
+        new_columns.push(series);
+
+        Ok(Self::new_unchecked(new_schema, new_columns, self.num_rows))
+    }
+
     fn eval_agg_expression(
         &self,
         agg_expr: &BoundAggExpr,
@@ -699,7 +714,7 @@ impl RecordBatch {
                     .collect::<DaftResult<Vec<_>>>()?;
                 func.evaluate(evaluated_inputs.as_slice(), func)
             }
-            Expr::ScalarFunction(func) => {
+            Expr::ScalarFn(ScalarFn::Builtin(func)) => {
                 let args = func.inputs
                     .iter()
                     .map(|e| {
@@ -710,15 +725,15 @@ impl RecordBatch {
 
                 func.udf.call(args)
             }
-            Expr::Literal(lit_value) => Ok(lit_value.to_series()),
+            Expr::Literal(lit_value) => Ok(lit_value.clone().into()),
             Expr::IfElse {
                 if_true,
                 if_false,
                 predicate,
             } => match predicate.as_ref() {
                 // TODO: move this into simplify expression
-                Expr::Literal(LiteralValue::Boolean(true)) => self.eval_expression(&BoundExpr::new_unchecked(if_true.clone())),
-                Expr::Literal(LiteralValue::Boolean(false)) => {
+                Expr::Literal(Literal::Boolean(true)) => self.eval_expression(&BoundExpr::new_unchecked(if_true.clone())),
+                Expr::Literal(Literal::Boolean(false)) => {
                     Ok(self.eval_expression(&BoundExpr::new_unchecked(if_false.clone()))?.rename(if_true.get_name(&self.schema)?))
                 }
                 _ => {
@@ -728,6 +743,10 @@ impl RecordBatch {
                     Ok(if_true_series.if_else(&if_false_series, &predicate_series)?)
                 }
             },
+            Expr::ScalarFn(ScalarFn::Python(python_udf)) => {
+                let args = python_udf.args().iter().map(|expr| self.eval_expression(&BoundExpr::new_unchecked(expr.clone()))).collect::<DaftResult<Vec<_>>>()?;
+                python_udf.call(args.as_slice()).map(|(s,_)| s)
+            }
             Expr::Subquery(_subquery) => Err(DaftError::ComputeError(
                 "Subquery should be optimized away before evaluation. This indicates a bug in the query optimizer.".to_string(),
             )),
@@ -904,15 +923,19 @@ impl RecordBatch {
     pub fn repr_html(&self) -> String {
         // Produces a <table> HTML element.
 
-        let mut res = "<table class=\"dataframe\">\n".to_string();
+        let num_columns = self.columns.len();
+
+        let mut res =
+            "<table class=\"dataframe\" style=\"table-layout: fixed; min-width: 100%\">\n"
+                .to_string();
 
         // Begin the header.
         res.push_str("<thead><tr>");
 
+        let header_style = format!("text-wrap: nowrap; width: calc(100vw / {}); min-width: 192px; overflow: hidden; text-overflow: ellipsis; text-align:left", num_columns);
+
         for field in self.schema.as_ref() {
-            res.push_str(
-                "<th style=\"text-wrap: nowrap; max-width:192px; overflow:auto; text-align:left\">",
-            );
+            res.push_str(&format!("<th style=\"{}\">", header_style));
             res.push_str(&html_escape::encode_text(&field.name));
             res.push_str("<br />");
             res.push_str(&html_escape::encode_text(&format!("{}", field.dtype)));
@@ -925,22 +948,24 @@ impl RecordBatch {
         // Begin the body.
         res.push_str("<tbody>\n");
 
+        let body_style = format!("text-align:left; width: calc(100vw / {}); min-width: 192px; max-height: 100px; overflow: hidden; text-overflow: ellipsis; word-wrap: break-word; overflow-y: auto", num_columns);
+
         let (head_rows, tail_rows) = if self.len() > 10 {
             (5, 5)
         } else {
             (self.len(), 0)
         };
 
-        let styled_td =
-            "<td><div style=\"text-align:left; max-width:192px; max-height:64px; overflow:auto\">";
-
         for i in 0..head_rows {
             // Begin row.
             res.push_str("<tr>");
 
-            for col in &*self.columns {
-                res.push_str(styled_td);
-                res.push_str(&html_value(col, i));
+            for (col_idx, col) in self.columns.iter().enumerate() {
+                res.push_str(&format!(
+                    "<td data-row=\"{}\" data-col=\"{}\"><div style=\"{}\">",
+                    i, col_idx, body_style
+                ));
+                res.push_str(&html_value(col, i, true));
                 res.push_str("</div></td>");
             }
 
@@ -950,7 +975,7 @@ impl RecordBatch {
 
         if tail_rows != 0 {
             res.push_str("<tr>");
-            for _ in &*self.columns {
+            for _ in 0..self.columns.len() {
                 res.push_str("<td>...</td>");
             }
             res.push_str("</tr>\n");
@@ -960,10 +985,13 @@ impl RecordBatch {
             // Begin row.
             res.push_str("<tr>");
 
-            for col in &*self.columns {
-                res.push_str(styled_td);
-                res.push_str(&html_value(col, i));
-                res.push_str("</td>");
+            for (col_idx, col) in self.columns.iter().enumerate() {
+                res.push_str(&format!(
+                    "<td data-row=\"{}\" data-col=\"{}\"><div style=\"{}\">",
+                    i, col_idx, body_style
+                ));
+                res.push_str(&html_value(col, i, true));
+                res.push_str("</div></td>");
             }
 
             // End row.
