@@ -7,7 +7,7 @@ use daft_core::prelude::Schema;
 use daft_dsl::{
     binary_op,
     expr::window::{WindowBoundary, WindowFrame},
-    functions::{FunctionArgs, ScalarFunction, ScalarUDF, FUNCTION_REGISTRY},
+    functions::{BuiltinScalarFn, FunctionArgs, ScalarUDF, FUNCTION_REGISTRY},
     unresolved_col, Expr, ExprRef, Operator, WindowExpr, WindowSpec,
 };
 use daft_session::Session;
@@ -99,7 +99,7 @@ impl SQLFunction for Arc<dyn ScalarUDF> {
             .iter()
             .map(|input| planner.plan_function_arg(input))
             .collect::<SQLPlannerResult<Vec<_>>>()?;
-        Ok(ScalarFunction {
+        Ok(BuiltinScalarFn {
             udf: self.clone(),
             inputs: daft_dsl::functions::FunctionArgs::try_new(inputs)?,
         }
@@ -269,7 +269,7 @@ impl SQLLiteral for i64 {
         Self: Sized,
     {
         expr.as_literal()
-            .and_then(daft_dsl::LiteralValue::as_i64)
+            .and_then(daft_core::lit::Literal::as_i64)
             .ok_or_else(|| PlannerError::invalid_operation("Expected an integer literal"))
     }
 }
@@ -291,7 +291,7 @@ impl SQLLiteral for bool {
         Self: Sized,
     {
         expr.as_literal()
-            .and_then(daft_dsl::LiteralValue::as_bool)
+            .and_then(daft_core::lit::Literal::as_bool)
             .ok_or_else(|| PlannerError::invalid_operation("Expected a boolean literal"))
     }
 }
@@ -309,7 +309,7 @@ impl<T: SQLLiteral> SQLLiteral for HashMap<String, T> {
         // get reference to struct
         let fields = expr
             .as_literal()
-            .and_then(daft_dsl::LiteralValue::as_struct)
+            .and_then(daft_core::lit::Literal::as_struct)
             .ok_or_else(|| PlannerError::invalid_operation("Expected a struct literal"))?;
         // add literals to new map
         let mut map = Self::new();
