@@ -8,7 +8,20 @@ import pytest
 from PIL import Image
 
 import daft
-from tests.utils import ANSI_ESCAPE, TD_STYLE, TH_STYLE
+from tests.utils import ANSI_ESCAPE
+
+
+def dataframe_td_style(num_cols: int) -> str:
+    return f'style="text-align:left; width: calc(100vw / {num_cols}); min-width: 192px; max-height: 100px; overflow: hidden; text-overflow: ellipsis; word-wrap: break-word; overflow-y: auto"'
+
+
+def dataframe_th_style(num_cols: int) -> str:
+    return f'style="text-wrap: nowrap; width: calc(100vw / {num_cols}); min-width: 192px; overflow: hidden; text-overflow: ellipsis; text-align:left"'
+
+
+def dataframe_th_style_schema() -> str:
+    return 'style="text-wrap: nowrap; max-width:192px; overflow:auto; text-align:left"'
+
 
 ROW_DIVIDER_REGEX = re.compile(r"╭─+┬*─*╮|├╌+┼*╌+┤")
 SHOWING_N_ROWS_REGEX = re.compile(r".*\(Showing first (\d+) of (\d+) rows\).*")
@@ -70,7 +83,10 @@ def parse_html_table(
     result = {}
     for table_key, table_values in pd_df.to_dict().items():
         name, dtype = table_key.split(" ")
-        result[name] = (dtype, [str(table_values[idx]) for idx in range(len(table_values))])
+        result[name] = (
+            dtype,
+            [str(table_values[idx]) for idx in range(len(table_values))],
+        )
     return result
 
 
@@ -96,6 +112,7 @@ def test_repr_with_non_default_preview_rows(make_df, num_preview_rows):
 
 
 def test_empty_df_repr(make_df):
+    num_cols = 2
     df = make_df({"A": [1, 2, 3], "B": ["a", "b", "c"]})
     df = df.where(df["A"] > 10)
     expected_data = {"A": ("Int64", []), "B": ("Utf8", [])}
@@ -105,7 +122,7 @@ def test_empty_df_repr(make_df):
         df._repr_html_()
         == f"""<div>
 <table class="dataframe">
-<thead><tr><th {TH_STYLE}>A<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
+<thead><tr><th {dataframe_th_style_schema()}>A<br />Int64</th><th {dataframe_th_style_schema()}>B<br />Utf8</th></tr></thead>
 </table>
 <small>(No data to display: Dataframe not materialized)</small>
 </div>"""
@@ -126,8 +143,8 @@ def test_empty_df_repr(make_df):
     assert (
         df._repr_html_()
         == f"""<div>
-<table class="dataframe">
-<thead><tr><th {TH_STYLE}>A<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
+<table class="dataframe" style="table-layout: fixed; min-width: 100%">
+<thead><tr><th {dataframe_th_style(num_cols)}>A<br />Int64</th><th {dataframe_th_style(num_cols)}>B<br />Utf8</th></tr></thead>
 <tbody>
 </tbody>
 </table>
@@ -137,6 +154,9 @@ def test_empty_df_repr(make_df):
 
 
 def test_alias_repr(make_df):
+    num_cols = 2
+    th_style = dataframe_th_style(num_cols)
+    td_style = dataframe_td_style(num_cols)
     df = make_df({"A": [1, 2, 3], "B": ["a", "b", "c"]})
     df = df.select(df["A"].alias("A2"), df["B"])
 
@@ -146,7 +166,7 @@ def test_alias_repr(make_df):
         df._repr_html_()
         == f"""<div>
 <table class="dataframe">
-<thead><tr><th {TH_STYLE}>A2<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
+<thead><tr><th {dataframe_th_style_schema()}>A2<br />Int64</th><th {dataframe_th_style_schema()}>B<br />Utf8</th></tr></thead>
 </table>
 <small>(No data to display: Dataframe not materialized)</small>
 </div>"""
@@ -168,12 +188,12 @@ def test_alias_repr(make_df):
     assert (
         df._repr_html_()
         == f"""<div>
-<table class="dataframe">
-<thead><tr><th {TH_STYLE}>A2<br />Int64</th><th {TH_STYLE}>B<br />Utf8</th></tr></thead>
+<table class="dataframe" style="table-layout: fixed; min-width: 100%">
+<thead><tr><th {th_style}>A2<br />Int64</th><th {th_style}>B<br />Utf8</th></tr></thead>
 <tbody>
-<tr><td><div {TD_STYLE}>1</div></td><td><div {TD_STYLE}>a</div></td></tr>
-<tr><td><div {TD_STYLE}>2</div></td><td><div {TD_STYLE}>b</div></td></tr>
-<tr><td><div {TD_STYLE}>3</div></td><td><div {TD_STYLE}>c</div></td></tr>
+<tr><td data-row="0" data-col="0"><div {td_style}>1</div></td><td data-row="0" data-col="1"><div {td_style}>a</div></td></tr>
+<tr><td data-row="1" data-col="0"><div {td_style}>2</div></td><td data-row="1" data-col="1"><div {td_style}>b</div></td></tr>
+<tr><td data-row="2" data-col="0"><div {td_style}>3</div></td><td data-row="2" data-col="1"><div {td_style}>c</div></td></tr>
 </tbody>
 </table>
 <small>(Showing first 3 of 3 rows)</small>
@@ -182,6 +202,9 @@ def test_alias_repr(make_df):
 
 
 def test_repr_with_unicode(make_df, data_source):
+    num_cols = 2
+    th_style = dataframe_th_style(num_cols)
+    td_style = dataframe_td_style(num_cols)
     df = make_df({"🔥": [1, 2, 3], "🦁": ["🔥a", "b🔥", "🦁🔥" * 60]})
     expected_data_unmaterialized = {"🔥": ("Int64", []), "🦁": ("Utf8", [])}
     expected_data_materialized = {
@@ -191,24 +214,28 @@ def test_repr_with_unicode(make_df, data_source):
         ),
         "🦁": (
             "Utf8",
-            ["🔥a", "b🔥", "🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁…"],
+            [
+                "🔥a",
+                "b🔥",
+                "🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁🔥🦁…",
+            ],
         ),
     }
 
     string_array = ["🔥a", "b🔥", "🦁🔥" * 60]  # we dont truncate for html
     expected_html_unmaterialized = f"""<div>
 <table class="dataframe">
-<thead><tr><th {TH_STYLE}>🔥<br />Int64</th><th {TH_STYLE}>🦁<br />Utf8</th></tr></thead>
+<thead><tr><th {dataframe_th_style_schema()}>🔥<br />Int64</th><th {dataframe_th_style_schema()}>🦁<br />Utf8</th></tr></thead>
 </table>
 <small>(No data to display: Dataframe not materialized)</small>
 </div>"""
     expected_html_materialized = f"""<div>
-<table class="dataframe">
-<thead><tr><th {TH_STYLE}>🔥<br />Int64</th><th {TH_STYLE}>🦁<br />Utf8</th></tr></thead>
+<table class="dataframe" style="table-layout: fixed; min-width: 100%">
+<thead><tr><th {th_style}>🔥<br />Int64</th><th {th_style}>🦁<br />Utf8</th></tr></thead>
 <tbody>
-<tr><td><div {TD_STYLE}>1</div></td><td><div {TD_STYLE}>{string_array[0]}</div></td></tr>
-<tr><td><div {TD_STYLE}>2</div></td><td><div {TD_STYLE}>{string_array[1]}</div></td></tr>
-<tr><td><div {TD_STYLE}>3</div></td><td><div {TD_STYLE}>{string_array[2]}</div></td></tr>
+<tr><td data-row="0" data-col="0"><div {td_style}>1</div></td><td data-row="0" data-col="1"><div {td_style}>{string_array[0]}</div></td></tr>
+<tr><td data-row="1" data-col="0"><div {td_style}>2</div></td><td data-row="1" data-col="1"><div {td_style}>{string_array[1]}</div></td></tr>
+<tr><td data-row="2" data-col="0"><div {td_style}>3</div></td><td data-row="2" data-col="1"><div {td_style}>{string_array[2]}</div></td></tr>
 </tbody>
 </table>
 <small>(Showing first 3 of 3 rows)</small>
@@ -237,9 +264,14 @@ def test_repr_with_html_string():
 
     non_html_table = df.__repr__()
     html_table = df._repr_html_()
+    td_style = dataframe_td_style(1)
+
     for i in range(3):
         assert f"<div>body{i}</div>" in non_html_table
-        assert f"<tr><td><div {TD_STYLE}>&lt;div&gt;body{i}&lt;/div&gt;</div></td></tr>" in html_table
+        assert (
+            f'<tr><td data-row="{i}" data-col="0"><div {td_style}>&lt;div&gt;body{i}&lt;/div&gt;</div></td></tr>'
+            in html_table
+        )
 
 
 class MyObj:
@@ -248,6 +280,8 @@ class MyObj:
 
 
 def test_repr_html_custom_hooks():
+    td_style = dataframe_td_style(3)
+
     img = Image.fromarray(np.ones((3, 3)).astype(np.uint8))
     arr = np.ones((3, 3))
 
@@ -291,7 +325,10 @@ def test_repr_html_custom_hooks():
     assert '<img style="max-height:128px;width:auto" src="data:image/png;base64,' in html_repr
 
     # Assert that numpy array viz hook correctly triggers in html repr
-    assert f"<td><div {TD_STYLE}>&ltnp.ndarray<br>shape=(3, 3)<br>dtype=float64&gt</div></td><td>" in html_repr
+    assert (
+        f'<td data-row="0" data-col="1"><div {td_style}>&ltnp.ndarray<br>shape=(3, 3)<br>dtype=float64&gt</div></td><td data-row="0" data-col="2">'
+        in html_repr
+    )
 
 
 def test_repr_empty_struct():
@@ -333,3 +370,41 @@ def test_repr_empty_struct():
 (Showing first 2 of 2 rows)"""
 
     assert ANSI_ESCAPE.sub("", str(df)) == expected_repr
+
+
+def test_interactive_html_with_record_batch():
+    """Test interactive HTML generation with a RecordBatch directly."""
+    from daft.dataframe.preview import PreviewFormatter
+
+    # Create a DataFrame and get its RecordBatch
+    df = daft.from_pydict({"A": [1, 2, 3], "B": ["a", "b", "c"]})
+    df.collect()
+
+    # Create a PreviewFormatter and generate interactive HTML
+    preview = df._preview
+    schema = df.schema()
+    formatter = PreviewFormatter(preview, schema)
+    html = formatter._generate_interactive_html()
+
+    # Extract the dataframe table part for exact testing
+    table_start = html.find('<table class="dataframe"')
+    table_end = html.find("</table>", table_start) + 8
+    table_html = html[table_start:table_end]
+
+    # Test exact match for the dataframe table
+    expected_table = f"""<table class="dataframe" style="table-layout: fixed; min-width: 100%">
+<thead><tr><th {dataframe_th_style(2)}>A<br />Int64</th><th {dataframe_th_style(2)}>B<br />Utf8</th></tr></thead>
+<tbody>
+<tr><td data-row="0" data-col="0"><div {dataframe_td_style(2)}>1</div></td><td data-row="0" data-col="1"><div {dataframe_td_style(2)}>a</div></td></tr>
+<tr><td data-row="1" data-col="0"><div {dataframe_td_style(2)}>2</div></td><td data-row="1" data-col="1"><div {dataframe_td_style(2)}>b</div></td></tr>
+<tr><td data-row="2" data-col="0"><div {dataframe_td_style(2)}>3</div></td><td data-row="2" data-col="1"><div {dataframe_td_style(2)}>c</div></td></tr>
+</tbody>
+</table>"""
+
+    assert table_html == expected_table
+
+    # Minimal checks for other components
+    assert "<style>" in html
+    assert "side-pane" in html
+    assert "showSidePane" in html
+    assert "serverUrl" in html
