@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import itertools
 from datetime import date, datetime, time, timedelta, timezone
+from functools import partial
 
 import numpy as np
 import pandas as pd
@@ -12,6 +13,7 @@ from daft.datatype import DataType, ImageMode, TimeUnit
 from daft.exceptions import DaftCoreException
 from daft.series import Series
 from tests.series import ARROW_FLOAT_TYPES, ARROW_INT_TYPES
+from tests.utils import random_numerical_embedding
 
 daft_int_types = [
     DataType.int8(),
@@ -785,6 +787,76 @@ def test_series_cast_fixed_shape_tensor_to_tensor() -> None:
 ### Embedding ###
 
 
+@pytest.mark.parametrize(
+    ["dtype", "size"],
+    [
+        (np.float32, 1024),
+        (np.float64, 512),
+        (np.int8, 2048),
+        (np.int16, 512),
+        (np.int32, 256),
+        (np.int64, 128),
+        (np.uint8, 2048),
+        (np.uint16, 512),
+        (np.uint32, 256),
+        (np.uint64, 128),
+    ],
+)
+def test_series_cast_fixed_shape_list_to_embedding(dtype: np.dtype, size: int):
+    daft_dtype = DataType.from_numpy_dtype(dtype)
+    make_array = partial(random_numerical_embedding, np.random.default_rng(), dtype, size)
+
+    data = [make_array() for _ in range(5)]
+    s = Series.from_pylist(data, dtype=DataType.fixed_size_list(daft_dtype, size))
+
+    target_dtype = DataType.embedding(daft_dtype, size)
+    t = s.cast(target_dtype)
+    assert len(t) == len(s)
+    assert t.datatype() == target_dtype, f"Expecting {target_dtype} but cast resulted in {t.datatype()}"
+    for i, (s_lst, t_arr) in enumerate(zip(s, t)):
+        assert len(s_lst) == len(t_arr)
+        assert len(s_lst) == size
+        assert isinstance(s_lst, list)
+        assert isinstance(t_arr, np.ndarray)
+        assert t_arr.shape == (size,)
+        assert (np.array(s_lst) == t_arr).all()
+
+
+@pytest.mark.parametrize(
+    ["dtype", "size"],
+    [
+        (np.float32, 1024),
+        (np.float64, 512),
+        (np.int8, 2048),
+        (np.int16, 512),
+        (np.int32, 256),
+        (np.int64, 128),
+        (np.uint8, 2048),
+        (np.uint16, 512),
+        (np.uint32, 256),
+        (np.uint64, 128),
+    ],
+)
+def test_series_cast_embedding_to_fixed_shape_list(dtype: np.dtype, size: int):
+    daft_dtype = DataType.from_numpy_dtype(dtype)
+    make_array = partial(random_numerical_embedding, np.random.default_rng(), dtype, size)
+
+    data = [make_array() for _ in range(5)]
+    s = Series.from_pylist(data, dtype=DataType.embedding(daft_dtype, size))
+
+    target_dtype = DataType.fixed_size_list(daft_dtype, size)
+    t = s.cast(target_dtype)
+    assert len(t) == len(s)
+    assert t.datatype() == target_dtype, f"Expecting {target_dtype} but cast resulted in {t.datatype()}"
+    for i, (s_arr, t_lst) in enumerate(zip(s, t)):
+        assert len(s_arr) == len(t_lst)
+        assert len(t_lst) == size
+        assert isinstance(s_arr, np.ndarray)
+        assert isinstance(t_lst, list)
+        assert s_arr.shape == (size,)
+        assert (np.array(t_lst) == s_arr).all()
+
+
 def test_series_cast_embedding_to_fixed_shape_tensor() -> None:
     shape = (4,)
     data = [
@@ -1315,3 +1387,23 @@ def test_cast_list_list_to_list_tensor():
     cast_to = DataType.list(DataType.tensor(DataType.int64(), shape=(4,)))
     s = s.cast(cast_to)
     assert s.datatype() == cast_to
+
+
+def test_cast_python_to_struct():
+    data = [{"a": 1, "b": True}, {"a": 3, "b": False}]
+    dtype = DataType.struct({"a": DataType.int64(), "b": DataType.bool()})
+
+    s = Series.from_pylist(data, pyobj="force")
+    s = s.cast(dtype)
+    assert s.datatype() == dtype
+    assert s.to_pylist() == data
+
+
+def test_cast_python_to_list_of_structs():
+    data = [[{"a": 1, "b": True}, {"a": 3, "b": False}]]
+    dtype = DataType.list(DataType.struct({"a": DataType.int64(), "b": DataType.bool()}))
+
+    s = Series.from_pylist(data, pyobj="force")
+    s = s.cast(dtype)
+    assert s.datatype() == dtype
+    assert s.to_pylist() == data
